/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef CM_H
#define CM_H


#include <mysql.h>
#include <NdbApi.hpp>
#include <mgmapi.h>



#include <string.h>

#define LOG_THREAD 0
#define MONITOR_THREAD 1
#define MAINT_THREAD 2
#define MYSQL_MON_THREAD 3
#define MYSQL_COLL_THREAD 4
#define NDB_COLL_THREAD 5
#define MYSQL_COLL_THREAD_ERR 6

#define MAX_THREADS 7

typedef struct{
  char connectstring[255];
  int save_clusterlog_hours;
  int save_statistics_hours;
  int port;
  char host[255];  
  char user[255];  
  char passwd[255];    
  char db[255];
  char socket[255];
} configparam_t;


enum start_mode {CS_NR, CS_INR};

/**
 * node_t does really only reflect the data nodes..
 * So this is very lazy.
 */
typedef struct 
{
  ndb_mgm_node_status status;  
  ndb_mgm_node_status prev_status;  
  ndb_mgm_node_type type;
  start_mode startmode;  
  int used_dm;
  int used_im;
  int total_dm;
  int total_im;
  int page_sz_dm;
  int page_sz_im;
  bool startIsOk;
  bool alarm;
} node_t;





enum enum_cluster_state {CS_STARTED, 
			 CS_NOT_STARTED, 
			 CS_DEGRADED,
			 CS_FAILURE, 
			 CS_SHUTTING_DOWN,
			 CS_RECOVERING,
			 CS_STARTING,
			 CS_UNKNOWN};

struct st_cluster_state_str
{
  const int state;
  const char* state_str;
};




/*
 * Both threads operates on this struct
 * and it is not yet made thread safe
 */

typedef struct
{
  int minNdbNodeId;
  int maxNdbNodeId;
  
  char connectstring[255];
  int noOfNodes;
  node_t nodes[255];
  enum_cluster_state current_state;
  enum_cluster_state previous_state;  
  bool block_restart;
  
} cluster_t;




typedef struct
{
  NdbMgmHandle handle;
  NdbLogEventHandle * le;
  char connectstring[255];
} cluster_conn_t;




typedef struct
{
  MYSQL mysql;
} mysql_t;




void cmon_exit(int code);



#endif
