#!/bin/bash
mycnf=/etc/mysql/my.cnf
bindir=/usr/local/mysql//mysql/bin/

libdir=/usr/local/mysql//mysql/lib/

TEMP=`getopt -o \?p:P:h:f:c: --long help,password:,hostname:,port:,config-file:,clusterid: -n 'cmon_add_mysqld.sh' -- "$@"`
eval set -- "$TEMP"
password=""
username=""
port=""
hostname=""
configfile=""
clusterid=""
help=""
while true ; do
    case "$1" in
        -h|--hostname) hostname="$2" ; shift 2 ;;
        -P|--port) port="$2" ; shift 2 ;;             
	-c|--clusterid) clusterid="$2" ; shift 2 ;;   
	-f|--config-file) configfile="$2" ; shift 2 ;;
        -\?|--help) help="1" ; shift ;;
        --) shift ; break ;;
        *) echo "Internal error!" ; exit 1 ;;
    esac
done
if [ -n "${help}" ]  ; then 
    echo "cmon_add_mysqld.sh" >&2 ; 
    echo "-h | --hostname=<hostname>- hostname of mysql server to add">&2; 
    echo "-P | --port=<port> - default 3306 - portnumber of mysql server to add">&2; 
    echo "-f | --config-file=<password> - cmon.conf file (default ../etc/cmon.conf)">&2; 
    echo "-f | --clusterid=<clusterid> - the clusterid (integer) to add this mysqld">&2; 
    echo "" 
    exit 1 ; 
fi


if [ "${hostname}" = "" ]; then
    echo "Must specify hostname of mysqld to add"
    exit;
fi


if [ "${clusterid}" = "" ]; then
    echo "clusterid is missing!"
    exit;
fi

if [ "${configfile}" = "" ]; then
    echo "Using default config file path ../etc/cmon.conf"
    configfile="../etc/cmon.conf"
fi


if [ "${port}" = "" ]; then
    port=3306    
fi


if [ ! -f ${configfile} ];
then
    echo "Could not find config file. Specify the file with --config-file"    
    exit
fi

source $configfile

source $configfile
if [ -z "${MYSQL_PASSWORD}" ]; then
    query="INSERT INTO cmon.mysql_server(cid,hostname,port,username) VALUES($clusterid, '$hostname', $port, 'cmon')"
else
    query="INSERT INTO cmon.mysql_server(cid,hostname,port,username,password) VALUES($clusterid, '$hostname', $port, 'cmon','$MYSQL_PASSWORD')"
fi

check="SELECT count(*) FROM cmon.mysql_server WHERE cid=$clusterid AND hostname='$hostname' AND port=$port"

if [ -z "${MYSQL_PASSWORD}" ]; then
    x=`$MYSQL_PATH/mysql --batch -N -u${MYSQL_USER} -h${MYSQL_HOST} -P${MYSQL_PORT} --execute "$check"`
else
    x=`$MYSQL_PATH/mysql --batch -N -u${MYSQL_USER} -p${MYSQL_PASSWORD} -h${MYSQL_HOST} -P${MYSQL_PORT} --execute  "$check"`

fi

if [ $x -gt 0 ]; then
  echo "MySQL Server already exists - nothing to add"
  exit
fi

if [ -z "${MYSQL_PASSWORD}" ]; then
    x=`$MYSQL_PATH/mysql -u${MYSQL_USER} -h${MYSQL_HOST} -P${MYSQL_PORT} --execute "$query"`
else
    x=`$MYSQL_PATH/mysql -u${MYSQL_USER} -p${MYSQL_PASSWORD} -h${MYSQL_HOST} -P${MYSQL_PORT} --execute "$query"`
fi


echo "Added MySQL Server"
echo ""
echo "You need to execute the following GRANTs on mysqld@${hostname} for CMON agent to be able to connect:"
echo ""
if [ -n "${MYSQL_PASSWORD}" ]; then
    echo "GRANT REPLICATION SLAVE, SUPER ON *.* TO '${MYSQL_USER}'@'${MYSQL_HOST} IDENTIFIED BY '${MYSQL_PASSWORD}';"
    echo "GRANT SELECT ON mysql.* TO '${MYSQL_USER}'@'${MYSQL_HOST} IDENTIFIED BY '${MYSQL_PASSWORD}';"
else
    echo "GRANT REPLICATION SLAVE, SUPER ON *.* TO '${MYSQL_USER}'@'${MYSQL_HOST}';"
    echo "GRANT SELECT ON mysql.* TO '${MYSQL_USER}'@'${MYSQL_HOST}';"
fi
echo ""

