#!/bin/bash

printf "\nThis script will install cmon, rrd, init.d scripts, and generate SQL scripts for GRANTs \n"

CLUSTERID=$1

DISTRIBUTION=""
TOOL=""
if [ -f /etc/SuSE-release ]; then
     DISTRIBUTION="suse"
     TOOL="chkconfig"
     WWWROOT=/var/www/html
  elif [ -f /etc/fedora-release ]; then
    DISTRIBUTION="fedora"
    TOOL="chkconfig"
    WWWROOT=/var/www/html
  elif [ -f /etc/debian_version ]; then
    DISTRIBUTION="debian"
    TOOL="update_rc.d"
    WWWROOT=/var/www/
  elif [ -f /etc/lsb-release ]; then
    DISTRIBUTION="ubuntu"
    TOOL="update_rc.d"
    WWWROOT=/var/www/
  elif [ -f /etc/redhat-release ]; then
    DISTRIBUTION=rhel
    TOOL="chkconfig"
    WWWROOT=/var/www/html/
else
    DISTRIBUTION="unknown"
    WWWROOT="''"
    printf "/etc/init.d script to start/stop cmon will not be installed because distribution is 'unknown'.\nInstall the etc/init.d/cmon script manually.\n"

fi

cp ../etc/cmon.conf.template ../etc/cmon.conf.t

if [ -z "${CLUSTERID}" ]; then
   CLUSTERID=1
fi
sed -ibak "s#@@CMON_CLUSTERID@@#$CLUSTERID#g" ../etc/cmon.conf.t

printf "Distribution: $DISTRIBUTION \n"

printf "\n**** MYSQL CONNECTION ****\n" 
printf "CMON and the  RRD scripts needs a mysql connection to speak mysql server holding the cmon database.\n"    
printf "Specify the BASEDIR where mysql is installed (default /usr/local/mysql/): "    
read MYSQLD_BASEDIR

if [ "$MYSQLD_BASEDIR"  = "" ]; then
    MYSQLD_BASEDIR="/usr/local/mysql"
fi

if [ ! -f $MYSQLD_BASEDIR/bin/mysql ]; then
    printf "ERROR: Can't find '$MYSQLD_BASEDIR/bin/mysql' - aborting install!\n"
    printf "Check where your BASEDIR is for the mysql installation!\n"
    exit
fi

sed -ibak "s#@@MYSQLD_BASEDIR@@#$MYSQLD_BASEDIR#g" ../etc/cmon.conf.t

thishost=`hostname`
printf "\nSpecify the hostname of the mysql server having the cmon database (default '${thishost}'): "
read MYSQLD_HOSTNAME

if [ "$MYSQLD_HOSTNAME"  = "" ]; then
    printf "No hostname specified - using default '${thishost}'\n"
    MYSQLD_HOSTNAME="${thishost}"
fi

sed -ibak "s#@@MYSQLD_HOST@@#$MYSQLD_HOSTNAME#g" ../etc/cmon.conf.t

printf "\nSpecify the port of the mysql server having the cmon database (default 3306): "    
read MYSQLD_PORT

if [ "$MYSQLD_PORT"  = "" ]; then
    printf "No port specified - using default 3306\n"
    MYSQLD_PORT="3306"
fi

sed -ibak "s#@@MYSQLD_PORT@@#$MYSQLD_PORT#g" ../etc/cmon.conf.t


printf "\nSpecify the password for the 'cmon' user (default no password): "    
read MYSQLD_PASSWORD

if [ "$MYSQLD_PASSWORD"  = "" ]; then
    printf "No password specified - using default (no password)\n"
    MYSQLD_PASSWORD=""
fi
sed -ibak "s#@@CMON_PASSWORD@@#$MYSQLD_PASSWORD#g" ../etc/cmon.conf.t

printf "\nSpecify the ndb-connectstring to the cluster (e.g, host_A;host_B): "
read NDB_CONNECTSTRING
if [ "$NDB_CONNECTSTRING"  = "" ]; then
    printf "ERROR: No ndb-connectstring set - aborting install.\n"
    exit
fi

printf "\n****  WWW interface ****\n" 

printf "The www files will be copied to ${WWWROOT}/cmon/\n"
printf "Specify the WWWROOT of your webserver (default $WWWROOT): "
read wroot

if [ "$wroot"  != "" ]; then
    WWWROOT=$wroot
fi

if [ -d $WWWROOT ]; then
    printf "Copying files to $WWWROOT .. \n"
    cp -r ../www/cmon $WWWROOT/
    if [ ! -d $WWWROOT/cmon ];
    then
	echo "Failed to copy $WWWROOT/cmon !"
	echo "Installation failed. Check permissions!"
	exit
    fi
    chmod 777 $WWWROOT/cmon/config    
else
    printf "\nERROR: WWWROOT $WWWROOT directory does not exists! Aborting install!\n"
    exit 
fi

sed -ibak "s#@@WWWROOT@@#$WWWROOT#g" ../etc/cmon.conf.t

printf "\n****  RRD ****\n" 

printf "Specify the full path to 'rrdtool' (default is /usr/bin/): "    
read RRD_PATH

if [ "$RRD_PATH"  = "" ]; then
    printf "No path to rrd specified - using default /usr/bin/\n"
    RRD_PATH="/usr/bin"
fi

RRD_TOOL=${RRD_PATH}/rrdtool
USE_RRD=1
if [ ! -f $RRD_TOOL ]; then
    printf "\nWARNING: Could not find rrdtool in $RRD_PATH!\n"
    printf "Make sure you have installed 'rrdtool'!\n"   
    USE_RRD=0
fi


if [ $USE_RRD = 1 ]; then

    sed -ibak "s#@@RRD_TOOL@@#$RRD_TOOL#g" ../etc/cmon.conf.t
    
    
    printf "\nThe rrdtool stores data files in a data directory.\n"    
    printf "Specify the full path to the data directory (about 20MB free space will be neeeded).\n"    
    printf "RRD data directory (default is /data/rrd/): "
    read RRD_DATADIR
    
    if [ "$RRD_DATADIR"  = "" ]; then
	printf "No RRD data directory specified - using default /data/rrd/\n"
	RRD_DATADIR="/data/rrd"
    fi

    
    if [ ! -d "$RRD_DATADIR" ]; then
	printf "The datadir $RRD__DATADIR does not exists and will be created now!\n"
	`mkdir -p $RRD_DATADIR`
	if [ ! -d $RRD_DATADIR ];
	then
	    echo "Failed to create $RRD_DATADIR !"
	    echo "Installation failed. Check permissions!"
	    exit
	fi
    fi
    
    if [ ! -d "$RRD_DATADIR" ]; then
	printf "\nERROR: The datadir $RRD_DATADIR could not be created - check if you have write permissions!\n"
	exit
    fi

    sed -ibak "s#@@RRD_DATADIR@@#$RRD_DATADIR#g" ../etc/cmon.conf.t
fi

printf "Saving ../etc/cmon.conf\n"
cp ../etc/cmon.conf.t ../etc/cmon.conf

if [ "$DISTRIBUTION" != "unknown" ]; then
    
    printf "\n**** INITD SCRIPTS ****\n" 
    printf "Do you want to install /etc/init.d/cmon (y/n)? : "
    read answer
    if [ "$answer" = "y" -o "$answer" = "Y" ]; then
	CMON_SBIN=`pwd`/../sbin/

	CMON_PID=""
	printf "Specify the directory where CMON should write its pidfile (default /var/run/): "    
	read CMON_PID
	
	if [ "$CMON_PID"  = "" ]; then
	    CMON_PID="/var/run/"
	fi

	cp ../etc/init.d/cmon.template ../etc/init.d/cmon.t
	sed -ibak "s#@@CMON_PID@@#$CMON_PID#g" ../etc/init.d/cmon.t
	sed -ibak "s#@@CMON_SBIN@@#$CMON_SBIN#g" ../etc/init.d/cmon.t
	sed -ibak "s#@@MYSQLD_BASEDIR@@#$MYSQLD_BASEDIR#g" ../etc/init.d/cmon.t
	sed -ibak "s#@@MYSQLD_PASSWORD@@#$MYSQLD_PASSWORD#g" ../etc/init.d/cmon.t
	sed -ibak "s#@@MYSQLD_HOSTNAME@@#$MYSQLD_HOSTNAME#g" ../etc/init.d/cmon.t
	sed -ibak "s#@@MYSQLD_PORT@@#$MYSQLD_PORT#g" ../etc/init.d/cmon.t
	sed -ibak "s#@@NDB_CONNECTSTRING@@#$NDB_CONNECTSTRING#g" ../etc/init.d/cmon.t
	CHK=""
	echo $TOOL
	if [ "${TOOL}" = "chkconfig" ]; then
	    if [ -f "/usr/sbin/chkconfig" ]; then 
		CHK=/usr/sbin/chkconfig
	    elif [ -f "/usr/bin/chkconfig" ]; then 
		CHK=/usr/bin/chkconfig
	    elif [ -f "/sbin/chkconfig" ]; then 
		CHK=/sbin/chkconfig
	    fi
	else
	    CHK=update-rc.d
	fi
	
	if [ -z "${CHK}" ]; then 
	    echo "Could not find $TOOL - exitting" 
	    exit
	fi
		
	printf "Saving configuration to ../etc/init.d/cmon\n"
	echo "Installing /etc/init.d/cmon"
	cp ../etc/init.d/cmon.t  /etc/init.d/cmon
	
	if [ ! -f /etc/init.d/cmon ];
	then
	    echo "Failed to copy /etc/init.d/cmon !"
	    echo "Installation failed. Check permissions!"
	    exit
	fi
	    
	
	chmod u+x /etc/init.d/cmon
	
	if [ "${TOOL}" = "chkconfig" ];
	then
	    $CHK --add cmon
	else
	    $CHK cmon defaults 99
	fi
	echo "Done - Installed init.d scripts"
	echo "Now you can start cmon with '/etc/init.d/cmon start'"
    fi
fi

if [ $USE_RRD = 1 ]; then
    printf "\n**** INSTALL CRONTAB ****\n" 
    printf "cron schedules jobs every 5 minutes to update the rrd database and generarate graphs for the web interface. You are recommended to install the cron jobs\n" 
    printf "Do you want to install cron jobs for cmon (y/n)? : "
    read answer
    if [ "$answer" = "y" -o "$answer" = "Y" ]; then
	BINDIR=`pwd`/../bin/
	CONFFILE=`pwd`/../etc/cmon.conf
	`crontab -l > /tmp/cmon_cron`
	echo "*/5 * * * * bash /$BINDIR/cmon_create_rrd.sh $CONFFILE" >> /tmp/cmon_cron
	echo "*/5 * * * * bash /$BINDIR/cmon_update_rrd.sh $CONFFILE" >> /tmp/cmon_cron
	echo "*/5 * * * * bash /$BINDIR/cmon_create_graphs.sh $CONFFILE" >> /tmp/cmon_cron
	crontab /tmp/cmon_cron
	rm /tmp/cmon_cron
    fi
fi


printf "You need to issue the following GRANTs before starting CMON:\n\n"

if [ "$MYSQLD_PASSWORD"  = "" ]; then
    printf "GRANT super, replication client ON *.* TO 'cmon'@'${MYSQLD_HOSTNAME}';\n";
    printf "GRANT select,update,insert,delete,create ON cmon.* TO 'cmon'@'${MYSQLD_HOSTNAME}';\n";
else
    printf "GRANT super, replication client ON *.* TO 'cmon'@'${MYSQLD_HOSTNAME}' IDENTIFIED BY '$MYSQLD_PASSWORD';\n";
    printf "GRANT select,update,insert,delete,create ON cmon.* TO 'cmon'@'${MYSQLD_HOSTNAME}' IDENTIFIED BY '$MYSQLD_PASSWORD';\n";
fi

if [ $USE_RRD = 1 ]; then
    printf "\n\nConfiguration now complete - you can edit the `pwd`/../etc/cmon.conf manually if you wish.\n\n"
else
    printf "\n\nConfiguration now complete - you can edit the `pwd`/../etc/cmon.conf manually if you wish.\n\n"
    printf "WARNING: RRD support was not installed as 'rrdtool' was not found. Re-run ./cmon_install.sh if you wish to change this.\n\n"
fi
