#Copyright (C) 2008 severalnines.com                                                                                
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or  
#(at your option) any later version. 
#This program is distributed in the hope that it will be useful,  
#but WITHOUT ANY WARRANTY; without even the implied warranty of 
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU General Public License 
#along with this program; if not, write to the Free Software
#
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#!/bin/bash

RRD_CONF="$1"

if [ -z ${RRD_CONF} ];
then
   echo "usage:"
   echo "./create_graphs <path to rrd.conf>"
exit
fi

source ${RRD_CONF}


function create_graph_cluster_stats()
{
  period=$1
  CLUSTER_ID=$2
  NODE_ID=$3

  filename=""
  imagename=""

  if [ -z ${NODE_ID} ];
  then
      filename=${RRD_DATADIR}/cluster_${CLUSTER_ID}_stats.rrd
      imagename=${IMAGEDIR}/cluster_${CLUSTER_ID}_stats_${period}.png
  else
      filename=${RRD_DATADIR}/cluster_${CLUSTER_ID}_node_${NODE_ID}_stats.rrd
      imagename=${IMAGEDIR}/cluster_${CLUSTER_ID}_node_${NODE_ID}_stats_${period}.png
  fi
   

  $RRD_TOOL graph $imagename \
	--step=300 \
	--start $period\
	--lower-limit=0\
	--rigid\
	DEF:trans=${filename}:transactions:AVERAGE:step=300  \
	DEF:ops=${filename}:operations:AVERAGE:step=300  \
	DEF:commits=${filename}:commits:AVERAGE:step=300  \
	DEF:writes=${filename}:writes:AVERAGE:step=300  \
	DEF:range_scans=${filename}:range_scans:AVERAGE:step=300  \
	DEF:scans=${filename}:scans:AVERAGE:step=300  \
	DEF:reads_nolock=${filename}:reads_nolock:AVERAGE:step=300  \
	DEF:reads_lock=${filename}:reads_lock:AVERAGE:step=300  \
	DEF:aborts=${filename}:aborts:AVERAGE:step=300  \
	DEF:concops=${filename}:concops:AVERAGE:step=300  \
     	COMMENT:"                " \
        COMMENT:"Current      "  \
        COMMENT:"Avg       "  \
        COMMENT:"Max       "  \
        COMMENT:"Min      \l"   \
	LINE1:trans#0000ff:"transactions  "  \
	GPRINT:trans:LAST:"%6.0lf    " \
	GPRINT:trans:AVERAGE:"%6.0lf    " \
	GPRINT:trans:MAX:"%6.0lf    " \
	GPRINT:trans:MIN:"%6.0lf    \l" \
	LINE1:ops#ff0000:"operations    "  \
	GPRINT:ops:LAST:"%6.0lf    " \
	GPRINT:ops:AVERAGE:"%6.0lf    " \
	GPRINT:ops:MAX:"%6.0lf    " \
	GPRINT:ops:MIN:"%6.0lf    \l" \
	LINE1:commits#00ff00:"commits       "  \
	GPRINT:commits:LAST:"%6.0lf    " \
	GPRINT:commits:AVERAGE:"%6.0lf    " \
	GPRINT:commits:MAX:"%6.0lf    " \
	GPRINT:commits:MIN:"%6.0lf    \l" \
	LINE1:writes#c0c0c0:"writes        "  \
	GPRINT:writes:LAST:"%6.0lf    " \
	GPRINT:writes:AVERAGE:"%6.0lf    " \
	GPRINT:writes:MAX:"%6.0lf    " \
	GPRINT:writes:MIN:"%6.0lf    \l" \
	LINE1:range_scans#52FFFF:"range scans   "  \
	GPRINT:range_scans:LAST:"%6.0lf    " \
	GPRINT:range_scans:AVERAGE:"%6.0lf    " \
	GPRINT:range_scans:MAX:"%6.0lf    " \
	GPRINT:range_scans:MIN:"%6.0lf    \l" \
	LINE1:scans#006600:"scans         "  \
	GPRINT:scans:LAST:"%6.0lf    " \
	GPRINT:scans:AVERAGE:"%6.0lf    " \
	GPRINT:scans:MAX:"%6.0lf    " \
	GPRINT:scans:MIN:"%6.0lf    \l" \
	LINE1:reads_lock#FFFF00:"reads lock    "  \
	GPRINT:reads_lock:LAST:"%6.0lf    " \
	GPRINT:reads_lock:AVERAGE:"%6.0lf    " \
	GPRINT:reads_lock:MAX:"%6.0lf    " \
	GPRINT:reads_lock:MIN:"%6.0lf    \l" \
	LINE1:reads_nolock#FF33FF:"reads w/o lock"  \
	GPRINT:reads_nolock:LAST:"%6.0lf    " \
	GPRINT:reads_nolock:AVERAGE:"%6.0lf    " \
	GPRINT:reads_nolock:MAX:"%6.0lf    " \
	GPRINT:reads_nolock:MIN:"%6.0lf    \l" \
        LINE1:aborts#000000:"aborts        "  \
	GPRINT:aborts:LAST:"%6.0lf    " \
	GPRINT:aborts:AVERAGE:"%6.0lf    " \
	GPRINT:aborts:MAX:"%6.0lf    " \
	GPRINT:aborts:MIN:"%6.0lf    \l" \
        LINE1:concops#00CCCC:"conc ops      "  \
	GPRINT:concops:LAST:"%6.0lf    " \
	GPRINT:concops:AVERAGE:"%6.0lf    " \
	GPRINT:concops:MAX:"%6.0lf    " \
	GPRINT:concops:MIN:"%6.0lf    \l" \
	-v "per sec" -t "mysql cluster statistics" 
}
  


function create_graph_cluster_mysql_stats()
{
  period=$1
  CLUSTER_ID=$2
  NODE_ID=$3

  filename=""
  imagename=""

  if [ -z ${NODE_ID} ];
  then
      filename=${RRD_DATADIR}/cluster_${CLUSTER_ID}_mysql_stats.rrd
      imagename=${IMAGEDIR}/cluster_${CLUSTER_ID}_mysql_stats_${period}.png
  else
      filename=${RRD_DATADIR}/cluster_${CLUSTER_ID}_mysql_${NODE_ID}_stats.rrd
      imagename=${IMAGEDIR}/cluster_${CLUSTER_ID}_mysql_${NODE_ID}_stats_${period}.png
  fi
   

  $RRD_TOOL graph $imagename \
	--step=300 \
	--start $period\
	--lower-limit=0\
	--rigid\
	DEF:select=${filename}:com_select:AVERAGE:step=300  \
	DEF:insert=${filename}:com_insert:AVERAGE:step=300  \
	DEF:update=${filename}:com_update:AVERAGE:step=300  \
	DEF:delete=${filename}:com_delete:AVERAGE:step=300  \
	DEF:replace=${filename}:com_replace:AVERAGE:step=300  \
     	COMMENT:"                " \
        COMMENT:"Current    "  \
        COMMENT:"Avg       "  \
        COMMENT:"Max       "  \
        COMMENT:"Min      \l"   \
	LINE1:select#0000ff:"COM_SELECT  "  \
	GPRINT:select:LAST:"%6.0lf    " \
	GPRINT:select:AVERAGE:"%6.0lf    " \
	GPRINT:select:MAX:"%6.0lf    " \
	GPRINT:select:MIN:"%6.0lf    \l" \
	LINE1:insert#ff0000:"COM_INSERT  "  \
	GPRINT:insert:LAST:"%6.0lf    " \
	GPRINT:insert:AVERAGE:"%6.0lf    " \
	GPRINT:insert:MAX:"%6.0lf    " \
	GPRINT:insert:MIN:"%6.0lf    \l" \
	LINE1:update#00ff00:"COM_UPDATE  "  \
	GPRINT:update:LAST:"%6.0lf    " \
	GPRINT:update:AVERAGE:"%6.0lf    " \
	GPRINT:update:MAX:"%6.0lf    " \
	GPRINT:update:MIN:"%6.0lf    \l" \
	LINE1:delete#ffff00:"COM_DELETE  "  \
	GPRINT:delete:LAST:"%6.0lf    " \
	GPRINT:delete:AVERAGE:"%6.0lf    " \
	GPRINT:delete:MAX:"%6.0lf    " \
	GPRINT:delete:MIN:"%6.0lf    \l" \
	LINE1:replace#ff00ff:"COM_REPLACE "  \
	GPRINT:replace:LAST:"%6.0lf    " \
	GPRINT:replace:AVERAGE:"%6.0lf    " \
	GPRINT:replace:MAX:"%6.0lf    " \
	GPRINT:replace:MIN:"%6.0lf    \l" \
	-v "per sec" -t "mysql statistics" 
}
  




function create_graph_cluster_memstats()
{
  
  period=$1
  CLUSTER_ID=$2
  NODE_ID=$3

  filename=""
  imagename=""

  if [ -z ${NODE_ID} ];
  then
      filename=${RRD_DATADIR}/cluster_${CLUSTER_ID}_memstats.rrd
      imagename=${IMAGEDIR}/cluster_${CLUSTER_ID}_mem_${period}.png
  else
      filename=${RRD_DATADIR}/cluster_${CLUSTER_ID}_node_${NODE_ID}_memstats.rrd
      imagename=${IMAGEDIR}/cluster_${CLUSTER_ID}_node_${NODE_ID}_mem_${period}.png
  fi

  $RRD_TOOL graph $imagename \
	--step=300 \
	--start $period\
	--lower-limit=0\
	--rigid\
	DEF:dm_used=${filename}:dm_used:AVERAGE:step=300  \
	DEF:dm_free=${filename}:dm_free:AVERAGE:step=300  \
	CDEF:dm_total=dm_used,dm_free,+ \
     	COMMENT:"                     " \
        COMMENT:"Current    "  \
        COMMENT:"Avg       "  \
        COMMENT:"Max              \n"  \
	AREA:dm_used#ff0000:"DataMemory (used)"  \
	GPRINT:dm_used:LAST:"%6.0lf    " \
	GPRINT:dm_used:AVERAGE:"%6.0lf    " \
	GPRINT:dm_used:MAX:"%6.0lf    \n" \
	STACK:dm_free#00ff00:"DataMemory (free)"  \
	GPRINT:dm_free:LAST:"%6.0lf    " \
	GPRINT:dm_free:AVERAGE:"%6.0lf    " \
	GPRINT:dm_free:MAX:"%6.0lf    \n" \
	GPRINT:dm_total:LAST:"  DataMemory (total)  %6.0lf" \
	GPRINT:dm_total:AVERAGE:"   %6.0lf     " \
	GPRINT:dm_total:MAX:"%6.0lf    \n" \
	-v "MB" -t "mysql cluster memory usage" 
}

if [ -z ${MYSQL_PASSWORD}  ]; then
    clusters=`$MYSQL_PATH/mysql -u${MYSQL_USER} -h${MYSQL_HOST}  -P${MYSQL_PORT}   --batch  --skip-column-names --execute   "SELECT concat_ws(' ', id) FROM cmon.cluster_state"`
else
    clusters=`$MYSQL_PATH/mysql -u${MYSQL_USER}  -h${MYSQL_HOST} -P${MYSQL_PORT} -p${MYSQL_PASSWORD}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', id) FROM cmon.cluster_state"`
fi

if [ -z "$clusters" ]; then
  echo "no clusters found - create_graph"
  exit
fi

for cluster in ${clusters}
do
   create_graph_cluster_stats "-1d" $cluster 
   create_graph_cluster_stats "-1w" $cluster
   create_graph_cluster_stats "-1m" $cluster
   create_graph_cluster_stats "-1y" $cluster

   create_graph_cluster_memstats "-1d"  $cluster
   create_graph_cluster_memstats "-1w"  $cluster
   create_graph_cluster_memstats "-1m"  $cluster
   create_graph_cluster_memstats "-1y"  $cluster
   
   create_graph_cluster_mysql_stats "-1d" $cluster
   create_graph_cluster_mysql_stats "-1w" $cluster
   create_graph_cluster_mysql_stats "-1m" $cluster
   create_graph_cluster_mysql_stats "-1y" $cluster
   
   if [ -z ${MYSQL_PASSWORD}  ]; then
       datanodes=`$MYSQL_PATH/mysql -u${MYSQL_USER}  -h${MYSQL_HOST} -P${MYSQL_PORT}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', nodeid) FROM cmon.node_state WHERE cid=${cluster} AND node_type='NDBD'"`
   else
       datanodes=`$MYSQL_PATH/mysql -u${MYSQL_USER} -h${MYSQL_HOST} -P${MYSQL_PORT} -p${MYSQL_PASSWORD} --batch  --skip-column-names --execute   "SELECT concat_ws(' ', nodeid) FROM cmon.node_state WHERE cid=${cluster} AND node_type='NDBD'"`
   fi
   

   for datanode in ${datanodes}
   do
       create_graph_cluster_stats "-1d" $cluster $datanode
       create_graph_cluster_stats "-1w" $cluster $datanode
       create_graph_cluster_stats "-1m" $cluster $datanode
       create_graph_cluster_stats "-1y" $cluster $datanode
       
       create_graph_cluster_memstats "-1d"  $cluster $datanode
       create_graph_cluster_memstats "-1w"  $cluster $datanode
       create_graph_cluster_memstats "-1m"  $cluster $datanode
       create_graph_cluster_memstats "-1y"  $cluster $datanode
   done
   if [ -z ${MYSQL_PASSWORD}  ]; then
       mysqlservers=`$MYSQL_PATH/mysql -u${MYSQL_USER} -P${MYSQL_PORT} -h${MYSQL_HOST}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', concat(hostname,'|',port) ) FROM cmon.mysql_server WHERE cid=${cluster}"`

   else
       mysqlservers=`$MYSQL_PATH/mysql -u${MYSQL_USER} -P${MYSQL_PORT} -p${MYSQL_PASSWORD} -h${MYSQL_HOST}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', concat(hostname,'|',port) ) FROM cmon.mysql_server WHERE cid=${cluster}"`
   fi

   for mysqlserver in ${mysqlservers}
   do
       create_graph_cluster_mysql_stats "-1d" $cluster $mysqlserver
       create_graph_cluster_mysql_stats "-1w" $cluster $mysqlserver
       create_graph_cluster_mysql_stats "-1m" $cluster $mysqlserver
       create_graph_cluster_mysql_stats "-1y" $cluster $mysqlserver
   done
done

