#Copyright (C) 2008 severalnines.com                                                                                
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or  
#(at your option) any later version. 
#This program is distributed in the hope that it will be useful,  
#but WITHOUT ANY WARRANTY; without even the implied warranty of 
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU General Public License 
#along with this program; if not, write to the Free Software
#
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#!/bin/bash

# Script to create rrd-file

RRD_CONF="$1"
if [ -z ${RRD_CONF} ];
then
   echo "usage:"
   echo "./create_rrd <path to rrd.conf> "
exit
fi


source $RRD_CONF

mkdir -p $RRD_DATADIR

if [ -z "${MYSQL_PASSWORD}" ]; then
    clusters=`$MYSQL_PATH/mysql -u${MYSQL_USER} -h${MYSQL_HOST} -P${MYSQL_PORT}   --batch  --skip-column-names --execute   "SELECT concat_ws(' ', id) FROM cmon.cluster_state"`
else
    clusters=`$MYSQL_PATH/mysql -u${MYSQL_USER} -h${MYSQL_HOST} -P${MYSQL_PORT} -p${MYSQL_PASSWORD}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', id) FROM cmon.cluster_state"`
fi
if [ -z "$clusters" ];
then
  echo "no clusters found - create_Rrd"
  exit
fi

for cluster in ${clusters}
do

    filename="cluster_${cluster}_stats.rrd"
    # Check i file already exists
    if [ ! -f "$RRD_DATADIR/$filename" ]
    then
   # File doesn't exist, create new rrd-file
	echo "Creating RRDtool DB for cluster statistics"
	$RRD_TOOL create $RRD_DATADIR/$filename \
	    --step 300 \
	    DS:transactions:DERIVE:600:0:U \
	    DS:operations:DERIVE:600:0:U \
	    DS:commits:DERIVE:600:0:U \
	    DS:writes:DERIVE:600:0:U \
	    DS:reads_lock:DERIVE:600:0:U \
	    DS:reads_nolock:DERIVE:600:0:U \
	    DS:aborts:DERIVE:600:0:U \
	    DS:range_scans:DERIVE:600:0:U \
	    DS:scans:DERIVE:600:0:U \
	    DS:concops:DERIVE:600:0:U \
	    RRA:AVERAGE:0.5:1:576		\
	    RRA:AVERAGE:0.5:6:672		\
	    RRA:AVERAGE:0.5:24:732		\
	    RRA:AVERAGE:0.5:288:1460	\
	    RRA:MAX:0.5:1:576		\
	    RRA:MAX:0.5:6:672		\
	    RRA:MAX:0.5:24:732		\
	    RRA:MAX:0.5:288:1460	\
	    RRA:MIN:0.5:1:576		\
	    RRA:MIN:0.5:6:672		\
	    RRA:MIN:0.5:24:732		\
	    RRA:MIN:0.5:288:1460	
	echo "Done!"
    else
	echo $RRD_DATADIR/$filename" already exists!"
    fi
    
    filename="cluster_${cluster}_memstats.rrd"
    if [ ! -f "$RRD_DATADIR/$filename" ]
    then 
    # File doesn't exist, create new rrd-file
	echo "Creating RRDtool DB for cluster mem usage"
	$RRD_TOOL create $RRD_DATADIR/$filename \
            --step 300 \
	    DS:dm_used:GAUGE:600:0:U \
	    DS:dm_free:GAUGE:600:0:U \
	    DS:im_used:GAUGE:600:0:U \
	    DS:im_free:GAUGE:600:0:U \
	    RRA:AVERAGE:0.5:1:576		\
	    RRA:AVERAGE:0.5:6:672		\
	    RRA:AVERAGE:0.5:24:732		\
	    RRA:AVERAGE:0.5:288:1460	
	echo "Done!"
    else
	echo "$RRD_DATADIR/$filename already exists!"
    fi    
    filename="cluster_${cluster}_mysql_stats.rrd"
    # Check i file already exists
    if [ ! -f "$RRD_DATADIR/$filename" ]
    then
   # File doesn't exist, create new rrd-file
	echo "Creating RRDtool DB for mysql statistics"
	$RRD_TOOL create $RRD_DATADIR/$filename \
	    --step 300 \
	    DS:com_delete:DERIVE:600:0:U \
	    DS:com_insert:DERIVE:600:0:U \
	    DS:com_replace:DERIVE:600:0:U \
	    DS:com_select:DERIVE:600:0:U \
	    DS:com_update:DERIVE:600:0:U \
	    RRA:AVERAGE:0.5:1:576		\
	    RRA:AVERAGE:0.5:6:672		\
	    RRA:AVERAGE:0.5:24:732		\
	    RRA:AVERAGE:0.5:288:1460	\
	    RRA:MAX:0.5:1:576		\
	    RRA:MAX:0.5:6:672		\
	    RRA:MAX:0.5:24:732		\
	    RRA:MAX:0.5:288:1460	\
	    RRA:MIN:0.5:1:576		\
	    RRA:MIN:0.5:6:672		\
	    RRA:MIN:0.5:24:732		\
	    RRA:MIN:0.5:288:1460	
	echo "Done!"
    else
	echo $RRD_DATADIR/$filename" already exists!"
    fi    

    if [ -z "${MYSQL_PASSWORD}"  ]; then
	datanodes=`$MYSQL_PATH/mysql -u${MYSQL_USER}  -h${MYSQL_HOST} -P${MYSQL_PORT}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', nodeid) FROM cmon.node_state WHERE cid=${cluster} AND node_type='NDBD'"`
    else
	datanodes=`$MYSQL_PATH/mysql -u${MYSQL_USER} -h${MYSQL_HOST} -P${MYSQL_PORT} -p${MYSQL_PASSWORD} --batch  --skip-column-names --execute   "SELECT concat_ws(' ', nodeid) FROM cmon.node_state WHERE cid=${cluster} AND node_type='NDBD'"`
    fi

   for datanode in ${datanodes}
   do
       filename="cluster_${cluster}_node_${datanode}_stats.rrd"
    # Check i file already exists
       if [ ! -f "$RRD_DATADIR/$filename" ]
       then
   # File doesn't exist, create new rrd-file
	   echo "Creating RRDtool DB for node statistics"
	   $RRD_TOOL create $RRD_DATADIR/$filename \
	       --step 300 \
	       DS:transactions:DERIVE:600:0:U \
	       DS:operations:DERIVE:600:0:U \
	       DS:commits:DERIVE:600:0:U \
	       DS:writes:DERIVE:600:0:U \
	       DS:reads_lock:DERIVE:600:0:U \
	       DS:reads_nolock:DERIVE:600:0:U \
	       DS:aborts:DERIVE:600:0:U \
	       DS:range_scans:DERIVE:600:0:U \
	       DS:scans:DERIVE:600:0:U \
	       DS:concops:DERIVE:600:0:U \
	       RRA:AVERAGE:0.5:1:576		\
	       RRA:AVERAGE:0.5:6:672		\
	       RRA:AVERAGE:0.5:24:732		\
	       RRA:AVERAGE:0.5:288:1460	\
	       RRA:MAX:0.5:1:576		\
	       RRA:MAX:0.5:6:672		\
	       RRA:MAX:0.5:24:732		\
	       RRA:MAX:0.5:288:1460	\
	       RRA:MIN:0.5:1:576		\
	       RRA:MIN:0.5:6:672		\
	       RRA:MIN:0.5:24:732		\
	       RRA:MIN:0.5:288:1460	
	   echo "Done!"
       else
	   echo $RRD_DATADIR/$filename" already exists!"
       fi
       filename="cluster_${cluster}_node_${datanode}_memstats.rrd"
       if [ ! -f "$RRD_DATADIR/$filename" ]
       then 
    # File doesn't exist, create new rrd-file
	   echo "Creating RRDtool DB for node mem usage"
	   $RRD_TOOL create $RRD_DATADIR/$filename \
               --step 300 \
	       DS:dm_used:GAUGE:600:0:U \
	       DS:dm_free:GAUGE:600:0:U \
	       DS:im_used:GAUGE:600:0:U \
	       DS:im_free:GAUGE:600:0:U \
	       RRA:AVERAGE:0.5:1:576		\
	       RRA:AVERAGE:0.5:6:672		\
	       RRA:AVERAGE:0.5:24:732		\
	       RRA:AVERAGE:0.5:288:1460	
	   echo "Done!"
       else
	   echo "$RRD_DATADIR/$filename already exists!"
       fi    
   done

   if [ -z "${MYSQL_PASSWORD}"  ]; then
       mysqlservers=`$MYSQL_PATH/mysql -u${MYSQL_USER} -P${MYSQL_PORT} -h${MYSQL_HOST}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', concat(hostname,'|',port) ) FROM cmon.mysql_server WHERE cid=${cluster}"`

   else
       mysqlservers=`$MYSQL_PATH/mysql -u${MYSQL_USER} -P${MYSQL_PORT} -p${MYSQL_PASSWORD} -h${MYSQL_HOST}  --batch  --skip-column-names --execute   "SELECT concat_ws(' ', concat(hostname,'|',port) ) FROM cmon.mysql_server WHERE cid=${cluster}"`
   fi
   
       
   for mysqlserver in ${mysqlservers}
   do
       hostname=`echo $mysqlserver | awk -F "|" '{print $1;}'`
       port=`echo $mysqlserver | awk -F "|" '{print $2;}'`
       filename="cluster_${cluster}_mysql_${mysqlserver}_stats.rrd"
    # Check i file already exists
       if [ ! -f "$RRD_DATADIR/$filename" ]
       then
   # File doesn't exist, create new rrd-file
	   echo "Creating RRDtool DB for mysql statistics"
	   $RRD_TOOL create $RRD_DATADIR/$filename \
	       --step 300 \
	       DS:com_delete:DERIVE:600:0:U \
	       DS:com_insert:DERIVE:600:0:U \
	       DS:com_replace:DERIVE:600:0:U \
	       DS:com_select:DERIVE:600:0:U \
	       DS:com_update:DERIVE:600:0:U \
	       RRA:AVERAGE:0.5:1:576		\
	       RRA:AVERAGE:0.5:6:672		\
	       RRA:AVERAGE:0.5:24:732		\
	       RRA:AVERAGE:0.5:288:1460	\
	       RRA:MAX:0.5:1:576		\
	       RRA:MAX:0.5:6:672		\
	       RRA:MAX:0.5:24:732		\
	       RRA:MAX:0.5:288:1460	\
	       RRA:MIN:0.5:1:576		\
	       RRA:MIN:0.5:6:672		\
	       RRA:MIN:0.5:24:732		\
	       RRA:MIN:0.5:288:1460	
	   echo "Done!"
       else
	   echo $RRD_DATADIR/$filename" already exists!"
       fi
   done
   
done
