

#include "alarm.h"
#include "cmon.h"
#include "log.h"
#include "handler.h"



extern cluster_t cluster_state;
extern int g_clusterid;

static struct st_cluster_state_str cluster_state_str[] = {
  {  CS_STARTED, "STARTED" },
  {  CS_NOT_STARTED, "NOT STARTED" },  
  {  CS_DEGRADED, "DEGRADED" },  
  {  CS_FAILURE, "FAILURE" },  
  {  CS_SHUTTING_DOWN, "SHUTTING DOWN" },   
  {  CS_RECOVERING, "RECOVERING" },   
  {  CS_STARTING, "STARTING" },   
  {  CS_UNKNOWN, "UNKNOWN" }
};


bool db_check_node_alarm(const int nodeid,   mysql_t * mysql)
{
  char msg[512];
  char query[128];
  snprintf(query,sizeof(query),"SELECT nodeid FROM alarm WHERE cid=%d AND nodeid=%d",g_clusterid, nodeid);

  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      
      return false;
    }
  else // query succeeded, process any data returned by it
    {
      MYSQL_RES * result = mysql_store_result(&mysql->mysql);
      if (result)  // there are rows
	{
	  
	  unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	  if(num_rows>0)
	    {
	      mysql_free_result(result);	 
	      return true;
	    }	  
	  mysql_free_result(result);	 
	}
    }
  return false;
}


bool db_check_cluster_alarm( mysql_t * mysql)
{
  char msg[512];
  char query[128];
  snprintf(query,sizeof(query),"SELECT nodeid FROM alarm WHERE cid=%d AND nodeid=0",g_clusterid);

  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      
      return false;
    }
  else // query succeeded, process any data returned by it
    {
      MYSQL_RES * result = mysql_store_result(&mysql->mysql);
      if (result)  // there are rows
	{
	  
	  unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	  if(num_rows>0)
	    {
	      mysql_free_result(result);	 
	      return true;
	    }	  
	  mysql_free_result(result);	 
	}
    }
  return false;
}




bool db_write_alarm( mysql_t * mysql,
		     const int cluster_id,  
		     const int nodeid, 
		     const int curr_cluster_state,
		     const int prev_cluster_state,
		     const char * severity, 
		     const char * component, 
		     const char * alarm_name, 
		     const char * exit_message,
		     const char * description,
		     const char * reco)
{
  char query[2048];
  char desc[256];
  char * hostname=(char*)malloc(128* sizeof(char));
  memset(hostname,0,sizeof(hostname));
  memset(query,0,sizeof(query));
  memset(desc,0,sizeof(desc));

  
  if(!db_get_ndbd_hostname(nodeid,hostname,mysql))
    {
      strcpy(hostname,"unknown");
    }
  
  
  
  unsigned long long alarm_id=db_get_alarm_id( mysql,
					       cluster_id,  
					       nodeid, 
					       component, 
					       alarm_name);
  if(alarm_id == 0)
    {
      snprintf(query, 
	       sizeof(query),
	       "REPLACE INTO alarm(cid,nodeid,component, alarm_name, severity, description," 
	       "hostname,cluster_state, exit_message, recommendation,report_ts)"
	       "VALUES (%d, %d, '%s', '%s', '%s', '%s','%s', '%s', '%s', '%s', now())",
	       cluster_id,
	       nodeid,
	       component,
	       alarm_name,
	       severity,
	       description,
	       hostname,
	       cluster_state_str[curr_cluster_state].state_str,
	       exit_message,
	       reco
	       );
      
      
      char msg[2048];
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  free(hostname);
	  return false;
	  
	}
      alarm_id = db_get_last_inserted_alarm_id(mysql);
      
      if(alarm_id>0)
	{
	  snprintf(query, 
		   sizeof(query),
		   "INSERT IGNORE INTO alarm_log(alarm_id,cid,nodeid,component, alarm_name, severity, description," 
		   "hostname,cluster_state, exit_message, recommendation, report_ts)"
		   "VALUES (%llu,%d, %d, '%s', '%s', '%s', '%s','%s', '%s', '%s', '%s', now())",
		   alarm_id,
		   cluster_id,
		   nodeid,
		   component,
		   alarm_name,
		   severity,
		   description,
		   hostname,
		   cluster_state_str[curr_cluster_state].state_str,
		   exit_message,
		   reco
		   );
	  
	  
	  char msg[2048];
	  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	    {
	      snprintf(msg, 
		       sizeof(msg),
		       "Query %s failed: Error: %s\n",
		       query,
		       mysql_error(&mysql->mysql));
	      log_message(LOG_WARNING, msg);
	      mysql_close(&mysql->mysql);      
	      free(hostname);
	      return false;
	      
	    } 
	  
	}
    }
  free(hostname);
  return true;
}


/**
   also checks if a alarm already exists
   * If returns id>0 then exissts
   * if returns id=0 then not exists
   */
unsigned long long db_get_alarm_id( mysql_t * mysql,
				    const int cluster_id,  
				    const int nodeid, 
				    const char * component, 
				    const char * alarm_name)
{
  
    char query[2048];
    
    if(alarm_name==0)
      {
	snprintf(query, 
		 sizeof(query),
		 "SELECT alarm_id FROM alarm WHERE cid=%d AND nodeid=%d AND component='%s'",	     
		 cluster_id,
		 nodeid,
		 component);	  
      }
    else
      {     
	snprintf(query, 
		 sizeof(query),
		 "SELECT alarm_id FROM alarm WHERE cid=%d AND nodeid=%d AND component='%s' AND alarm_name='%s'",	     
		 cluster_id,
		 nodeid,
		 component,
		 alarm_name);	  
      }
    
    unsigned long long alarm_id=0;
    
    if(mysql_real_query(&mysql->mysql, query, strlen(query)))
      {  
	char msg[1024];
	snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
		 query,
		 mysql_error(&mysql->mysql));
	log_message(LOG_WARNING, msg);
	
	return 0;
      }
    else // query succeeded, process any data returned by it
      {
	MYSQL_RES * result = mysql_store_result(&mysql->mysql);
	if (result)  // there are rows
	  {	    
	    unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	    MYSQL_ROW row;
	    
	    if(num_rows>0)
	      {
		while ((row = mysql_fetch_row(result)))
		  {
		    alarm_id=atoll(row[0]);
		  }
	      }	  
	    mysql_free_result(result);	 
	  }
      }
    return alarm_id;
}






/**
   get last inserted alarm id... using LAST_INSERT_ID(..).. :(
 */
unsigned long long db_get_last_inserted_alarm_id( mysql_t * mysql)
{
  
    char query[2048];

    snprintf(query, 
	     sizeof(query),
	     "SELECT LAST_INSERT_ID()");
  
    unsigned long long alarm_id=0;
    
    if(mysql_real_query(&mysql->mysql, query, strlen(query)))
      {  
	char msg[1024];
	snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
		 query,
		 mysql_error(&mysql->mysql));
	log_message(LOG_WARNING, msg);
	
	return 0;
      }
    else // query succeeded, process any data returned by it
      {
	MYSQL_RES * result = mysql_store_result(&mysql->mysql);
	if (result)  // there are rows
	  {	    
	    unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	    MYSQL_ROW row;
	    
	    if(num_rows>0)
	      {
		while ((row = mysql_fetch_row(result)))
		  {
		    alarm_id=atoll(row[0]);
		  }
	      }	  
	    mysql_free_result(result);	 
	  }
      }
    return alarm_id;
}




bool db_cluster_clear_alarm( mysql_t * mysql)
{
  
  if(db_check_cluster_alarm(mysql))
    {  
      char query[512];
      char msg[1024];
      
      snprintf(query,
	       sizeof(query),
	       "DELETE FROM alarm WHERE cid=%d and nodeid=0 and component='CLUSTER'\n",
	       g_clusterid);
      
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  return false;
	  
	}
    }
    return true;
}



bool db_node_clear_alarm( mysql_t * mysql,
			  const int nodeid,
			  const char * node_type,
			  const char * severity,
			  const char * alarm_name )
{
  
  unsigned long long alarm_id=db_get_alarm_id( mysql,
					       g_clusterid,  
					       nodeid, 
					       node_type, 
					       alarm_name);

  if(alarm_id>0) //there is an alarm to clear
    {  
      
      char * hostname=(char*)malloc(128* sizeof(char));
      memset(hostname,0,sizeof(hostname));
  
      if(!db_get_ndbd_hostname(nodeid,hostname,mysql))
	strcpy(hostname,"unknown");
      

      char query[512];
      char msg[1024];
      

      snprintf(query,
	       sizeof(query),
	       "DELETE FROM alarm WHERE cid=%d and nodeid=%d\n",
	       g_clusterid,
	       nodeid);
      
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  return false;
	  
	}
      snprintf(query, 
	       sizeof(query),
	       "INSERT IGNORE INTO alarm_log(alarm_id,cid,nodeid,component, alarm_name, severity, description," 
	       "hostname,cluster_state, exit_message, recommendation, report_ts)"
	       "VALUES (%llu,%d, %d, '%s', 'CLEARED ALARM', 'INFO', '%s','%s', '%s', '%s', '%s', now())",
	       alarm_id,
	       g_clusterid,
	       nodeid,
	       node_type,
	       "CLEARED NODE DISCONNECTED ALARM",
	       hostname,
	       cluster_state_str[cluster_state.current_state].state_str,
	       "",
	       ""
	       );
      
      
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  free(hostname);
	  return false;
	  
	} 
      
    }
    return true;
}



bool db_write_resource_alarm( mysql_t * mysql,
			      const int nodeid, 		     
			      const char * severity, 
			      const char * resource, 
			      const char * alarm_name, 
			      const char * node_type,
			      const char * description,
			      const char * reco)
{
  
  return db_write_alarm( mysql,
			 g_clusterid,
			 nodeid,
			 cluster_state.current_state,
			 cluster_state.previous_state,
			 severity, 
			 resource, 
			 alarm_name, 
			 "",
			 description,
			 reco);
}

bool db_clear_resource_alarm( mysql_t * mysql,
			      const int nodeid, 		     
			      const char * resource,
			      const char * descr)
{
  unsigned long long alarm_id=db_get_alarm_id( mysql,
					       g_clusterid,  
					       nodeid, 
					       resource,
					       0);

  if(alarm_id>0) //there is an alarm to clear
    {  
      
      char * hostname=(char*)malloc(128* sizeof(char));
      memset(hostname,0,sizeof(hostname));
  
      if(!db_get_ndbd_hostname(nodeid,hostname,mysql))
	strcpy(hostname,"unknown");
      

      char query[512];
      char msg[1024];
      
      
      snprintf(query,
	       sizeof(query),
	       "DELETE FROM alarm WHERE alarm_id=%llu",
	       alarm_id);
      
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  return false;
	  
	}
      snprintf(query, 
	       sizeof(query),
	       "INSERT INTO alarm_log(alarm_id,cid,nodeid,component, alarm_name, severity, description," 
	       "hostname,cluster_state, exit_message, recommendation, report_ts)"
	       "VALUES (%llu,%d, %d, '%s', 'CLEARED RESOUCE ALARM', 'INFO', '%s','%s', '%s', '%s', '%s', now())",
	       alarm_id,
	       g_clusterid,
	       nodeid,
	       resource,
	       descr,
	       hostname,
	       cluster_state_str[cluster_state.current_state].state_str,
	       "",
	       ""
	       );
      
      
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  free(hostname);
	  return false;
	  
	} 
      
    }
  return true;
}

#if 0
bool db_clear_alarm_component( mysql_t * mysql,
			       const int nodeid,
			       const char * component,
			       const char * node_type)
{
  char query[512];
  char msg[1024];
  //  if(db_check_node_alarm(nodeid,mysql))
  // {  
      
      
      snprintf(query,
	       sizeof(query),
	       "INSERT INTO alarm_log(cid,nodeid,component,alarm_name,alarm_count,"
	       "alarm_sent,alarm_sent_count,cluster_state,severity,exit_message,"
	       "description,recommendation,hostname)  "
	       "SELECT cid,nodeid,component,alarm_name,alarm_count,alarm_sent,"
	       "alarm_sent_count,cluster_state,severity,exit_message,description,"
	       "recommendation,hostname FROM cmon.alarm  WHERE cid=%d and nodeid=%d and component='%s'\n",
	       g_clusterid,
	       0,
	       component);
      
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
	       "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  return false;
	}
      
      snprintf(query,
	       sizeof(query),
	       "DELETE FROM alarm WHERE cid=%d and nodeid=0 and component='%s'\n",
	       g_clusterid,
	       component);
      
      if(mysql_real_query(&mysql->mysql, query, strlen(query)))
	{
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s\n",
		   query,
		   mysql_error(&mysql->mysql));
	  log_message(LOG_WARNING, msg);
	  mysql_close(&mysql->mysql);      
	  return false;
	  
	}
      
      return true;
}
    
#endif

bool db_update_alarm_count( mysql_t * mysql,
			    const int nodeid)
{
  char query[128];
  snprintf(query,
	   sizeof(query),
	   "UPDATE alarm SET alarm_count=alarm_count+1, report_ts=now() WHERE cid=%d and nodeid=%d\n",
	   g_clusterid,
	   nodeid);
  
  char msg[256];
  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg, 
	       sizeof(msg),
	       "Query %s failed: Error: %s\n",
	       query,
	       mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      mysql_close(&mysql->mysql);      
      return false;
      
    }
  return true;
}




bool db_update_alarm_error( mysql_t * mysql,
			    const int nodeid,
			    const char * exit_msg,
			    const char * desc,
			    const char * rec)
{
  
  char query[512];
  snprintf(query,
	   sizeof(query),
	   "UPDATE alarm SET alarm_count=alarm_count+1, exit_message='%s',"
	   "description='%s',"
	   "recommendation='%s',"
	   "report_ts=now() WHERE cid=%d and nodeid=%d and alarm_name='DATA NODE FAILED'",
	  exit_msg,
	  desc,
	  rec,
	  g_clusterid,
	  nodeid);
  
  printf("query=%s\n", query);
  char msg[1024];
  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg, 
	       sizeof(msg),
	       "Query %s failed: Error: %s\n",
	       query,
	       mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      mysql_close(&mysql->mysql);      
      return false;

    }
  return true;
  
}

