/* Copyright (C) 2008 severalnines.com
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <mysql.h>
#include <NdbApi.hpp>
#include <mgmapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <string.h>
#include <getopt.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <sys/stat.h>


#include "monitor.h"
#include "logevent.h"
#include "statistics.h"
#include "maint.h"
#include "mysql_mon.h"
#include "mysql_coll.h"
#include "ndb_coll.h"
#include "connect.h"
#include "handler.h"

#include "schema.h"
#include "../config.h"

#include "cmon.h"
#include "log.h"


#define LOCK_FILE	"cmon.pid"



char g_running_dir[256];

struct  handler_list_t *  g_handler_list;
int g_plugin_count;  
Ndb_cluster_connection  * g_ndb_conn;

configparam_t params;
cluster_t cluster_state;
cluster_conn_t cluster_conn[2];
mysql_t mysql_conn[MAX_THREADS];


extern bool g_nodaemon;
extern bool g_log;
extern char g_logfilename[255];
extern pthread_mutex_t mut_log;

int g_clusterid;

void load_plugins(const char * plugin_list, char  ** argv, int argc)
{
//#if HAVE_DLFCN_H
#if 0
	
	const char * plugin=NULL;
	const char * delim=",";
	char  msg[256];
	memset(msg,0,sizeof(msg));
	g_plugin_count=0;
	g_handler_list=(handler_list_t*)malloc(sizeof(struct handler_list_t));
	
	while((plugin=strtok((char*)plugin_list, delim))!=NULL)
    {      
		g_handler_list=(handler_list_t*)realloc(g_handler_list, 
												sizeof(struct handler_list_t));
		
		if(g_handler_list==0)
			cmon_exit(errno);
		
		plugin_list=NULL;
		
		g_handler_list[g_plugin_count].handle=(void*)dlopen(plugin,RTLD_LAZY);
		strcpy(g_handler_list[g_plugin_count].name, plugin);
		
		if(g_handler_list[g_plugin_count].handle == NULL)
		{
			sprintf(msg,"Plugin %s - could not be loaded (check path,"
					"spelling, LD_LIBRARY_PATH, etc)",
					g_handler_list[g_plugin_count].name);
			log_message(LOG_WARNING,msg);
			cmon_exit(-1);
		}
		
		if(handler_initialize((const char **)argv, (const int)argc, g_plugin_count))
		{
			memset(msg,0,sizeof(msg));
			sprintf(msg,"Success - Loaded and intialized plugin - %s",
					g_handler_list[g_plugin_count].name);
			log_message(LOG_INFO,msg);
			g_plugin_count++;	  
		}	
		else
		{
			g_handler_list[g_plugin_count].handle=0;
			
			sprintf(msg,
					"Loaded plugin FAILED - %s. handler_initialize(..) failed",
					g_handler_list[g_plugin_count].name);		  
			log_message(LOG_ERR,msg);
			
		}
    }
#else
	log_message(LOG_INFO,"plugin interface disabled");
#endif
} 


void cmon_exit(int code)
{
	char msg[128];
	sprintf(msg,"Critical error (mysql error code %d) occured - shutting down",
			code);
	log_message(LOG_ERR,msg);
	exit(-1);
}



void signal_handler(int sig)
{
	switch(sig) {
		case SIGHUP:
			log_message(LOG_INFO,"Hangup signal catched");
			break;
		case SIGTERM:
			log_message(LOG_INFO,"Terminate signal catched ... shutting down.. bye!");
			exit(0);
			break;
	}
}

void daemonize()
{
	int i,lfp;
	char str[10];
	if(getppid()==1) return; /* already a daemon */
	i=fork();
	if (i<0) exit(1); /* fork error */
	if (i>0) exit(0); /* parent exits */
	/* child (daemon) continues */
	setsid(); /* obtain a new process group */
	for (i=getdtablesize();i>=0;--i) close(i); /* close all descriptors */
	i=open("/dev/null",O_RDWR); 
	
	 /* handle standard I/O */
	if(dup(i) <0)
	  exit(-1);
	if(dup(i) <0)
	  exit(-1);
	umask(027); /* set newly created file permissions */
	if(chdir(g_running_dir) < 0) exit(-1); /* change running directory */
	lfp=open(LOCK_FILE,O_RDWR|O_CREAT,0640);
	if (lfp<0) exit(1); /* can not open */
	if (lockf(lfp,F_TLOCK,0)<0) exit(0); /* can not lock */
	/* first instance continues */
	sprintf(str,"%d\n",getpid());
	if(write(lfp,str,strlen(str))<0) exit(-1); /* record pid to lockfile */
	signal(SIGCHLD,SIG_IGN); /* ignore child */
	signal(SIGTSTP,SIG_IGN); /* ignore tty signals */
	signal(SIGTTOU,SIG_IGN);
	signal(SIGTTIN,SIG_IGN);
	signal(SIGHUP,signal_handler); /* catch hangup signal */
	signal(SIGTERM,signal_handler); /* catch kill signal */
}





int option(int argc, char** argv)
{
	int c;
	g_log=false;
	while (1)
    {
		static struct option long_options[] =
		{
			{"help", 0, 0, '?'},
			{"mysqlhost", 1, 0, 'h'},
			{"mysqlport", 1, 0, 'P'},
			{"mysqlpasswd", 1, 0, 'p'},
			{"clusterid", 1, 0, 'i'},
			//	  {"clustername", 1, 0, 'm'},
			{"plugins", 1, 0, 'w'},
			{"logfile", 1, 0, 'l'},
			{"pidfile", 1, 0, 'f'},
			{"nodaemon", 0, 0, 'n'},
			{"ndb-connectstring", 1, 0, 'c'},
			{"savetime-clusterlog", 1, 0, 'x'},
			{"savetime-statistics", 1, 0, 't'},
			{0, 0, 0, 0}
		};
		/* getopt_long stores the option index here.   */
		int option_index = 0;
		
		c = getopt_long (argc, argv, "?nh:d:p:c:x:t:P:l:w:i:f:",
						 long_options, &option_index);
		
		/* Detect the end of the options.   */
		if (c == -1)
		{
			break;
		}
		
		
		switch (c)
		{
			case 0:
				/* If this option set a flag, do nothing else now.   */
				if (long_options[option_index].flag != 0)
					break;
				printf ("option %s", long_options[option_index].name);
				if (optarg)
					printf (" with arg %s", optarg);
				printf ("\n");
				
				break;
				
				/*	case 'n':
				 strcpy(g_clustername,optarg);
				 break;
				 */
			case 'i':
				g_clusterid=atoi(optarg);
				break;
				
			case 'c':
				memset(params.connectstring,0, sizeof(params.connectstring));
				strcpy(params.connectstring,optarg);
				break;
				
			case 'x':
				params.save_clusterlog_hours = atoi(optarg);
				break;
			case 't':
				params.save_statistics_hours = atoi(optarg);
				break;
			case 'p':
				memset(params.passwd,0, sizeof(params.passwd));
				strcpy(params.passwd,optarg);
				break;
				
			case 'f':
				strcpy(g_running_dir,optarg);
				break;
				
				
			case 'P':
				params.port = atoi(optarg);
				break;
			case 'l':
				memset(g_logfilename,0, sizeof(g_logfilename));
				strcpy(g_logfilename,optarg);
				g_log=true;
				break;
			case 'h':
				memset(params.host,0, sizeof(params.host));
				strncpy(params.host,optarg, strlen(optarg));
				break;
			case 'w':
				load_plugins(optarg, argv, argc);
				break;
			case '?':
			{
				printf("  -? --help\t\t\tprints this information\n");
				printf("  -n --nodaemon\t\t\tstarts in nodaemon mode\n");
				printf("  -c --ndb-connectstring=<hostname>\thostname where ndb_mgmd is running, default 'localhost'\n");
				printf("  -h --mysqlhost=<hostname>\t\thostname where mysqld is running, default 'localhost'\n");
				printf("  -P --mysqlport=<port>\t\t\tport mysqld listens on, default 3306\n");
				printf("  -p --mysqlpasswd=<passwd>\t\tpassword for user to connect to mysqld, default ''\n");
				printf("\n");
				printf("  -f --pidfile=<path>\t\twhere cmon writes the pidfile cmon.pid) (default /tmp/)\n");
				printf("  -l --logfile=<path/logfile>\t\tonly needed to be set if you want to log to another place than syslog (cmon will by default log to syslog). \n");
				printf("  --savetime-clusterlog=<hours>\thow old cluster_log 'monitor' will keep, default 24 (hours)\n");
				printf("  --savetime-statistics=<hours>\thow old statistics 'monitor' will keep, default 24 (hours)\n");
				exit(-1);
				break;
			}
				
			case 'n':
			{
				g_nodaemon=true;
				break;
			}
			default:
				printf("Wrong options given. Try '-?' for help\n");
				exit(-1);
				break;
		}
    }
	return 0;  
	
}

bool check_cluster_exists()
{
	sleep(1);
	MYSQL  mysql;  
	if(!connectMySQL2(&mysql, params.db))
    {
		log_message_force(LOG_INFO, "Could not register managed cluster - problem connecting to mysql.",1);
		fprintf(stderr,"Could not register managed cluster - problem connecting to mysql.\n");
		cmon_exit(-1);
    }  
	bool found=false;
	
	char sqlbuf[255];
	memset(sqlbuf,0,sizeof(sqlbuf));
	sprintf( sqlbuf,"SELECT id,connectstring FROM cluster WHERE connectstring='%s'", params.connectstring);
	int a=strlen(sqlbuf);
	if(mysql_real_query( &mysql, sqlbuf, a ))
    {
		
		char msg[512];
		memset(msg,0,sizeof(msg));
		sprintf(msg, "Query %s failed: Error (%d): %s",
				sqlbuf,
				mysql_errno(&mysql),
				mysql_error(&mysql));
		
		log_message_force(LOG_ERR, msg,1);
		mysql_close(&mysql);
		log_message_force(LOG_INFO, "Could not register managed cluster - problem connecting to mysql.",1);
		fprintf(stderr,"Could not register managed cluster - problem connecting to mysql.\n");
		cmon_exit(-1);
    }       
	else // query succeeded, process any data returned by it
	{
		MYSQL_RES * result = mysql_store_result(&mysql);
		if (result)  // there are rows
	    {
			MYSQL_ROW row;
			while ((row = mysql_fetch_row(result)))
			{
				if(g_clusterid!=atoi(row[0]))
				{
					fprintf(stderr, 
							"The cluster (id=%d) already registered with connectstring='%s', but you have provided cluster id=%d\n",
							atoi(row[0]),
							params.connectstring,
							g_clusterid);
					fprintf(stderr,"You should probably start cmon with cluster id=%d  - exitting..\n", atoi(row[0]));
					exit(-1);
					
				}
				found=true;
			}	       
			mysql_free_result(result);
	    }
	} 
	mysql_close(&mysql);
	return found;
}

bool add_cluster()
{
	MYSQL  mysql;  
	if(!connectMySQL2(&mysql, params.db))
    {
		log_message_force(LOG_INFO, "Could not register managed cluster - problem connecting to mysql.",1);
		cmon_exit(-1);
    }  
	char sqlbuf[255];
	memset(sqlbuf,0,sizeof(sqlbuf));
	sprintf( sqlbuf,"INSERT INTO cluster(id, connectstring) VALUES(%d,'%s')", g_clusterid,params.connectstring);
	
	if(mysql_real_query(&mysql, sqlbuf, strlen(sqlbuf)))
    {
		
		fprintf(stderr, "mysql error message: %s", mysql_error(&mysql));
		
		
		return false;
    }
    //  if(!executeQuery2(sqlbuf,&mysql))
	
	/*
	 memset(sqlbuf,0,sizeof(sqlbuf));
	 sprintf( sqlbuf,"SELECT last_insert_id()");
	 bool found=false;
	 
	 int a=strlen(sqlbuf);
	 if(mysql_real_query( &mysql, sqlbuf, a ))
	 {
	 char msg[512];
	 memset(msg,0,sizeof(msg));
	 sprintf(msg, "Query %s failed: Error: %s",
	 sqlbuf,
	 mysql_error(&mysql));
	 log_message_force(LOG_ERR, msg,1);
	 mysql_close(&mysql);
	 log_message_force(LOG_INFO, "Could not register managed cluster - problem connecting to mysql.",1);
	 cmon_exit(-1);
	 }       
	 else // query succeeded, process any data returned by it
	 {
	 MYSQL_RES * result = mysql_store_result(&mysql);
	 if (result)  // there are rows
	 {
	 MYSQL_ROW row;
	 unsigned int num_fields;
	 
	 num_fields = mysql_num_fields(result);
	 while ((row = mysql_fetch_row(result)))
	 {
	 printf("Cluster = %d\n", atoi(row[0]));
	 g_clusterid=atoi(row[0]);
	 found=true;
	 }	       
	 mysql_free_result(result);
	 }
	 } 
	 mysql_close(&mysql);
	 return found;
	 */
	mysql_close(&mysql);
	return true;
}



int register_cmon()
{
  fprintf(stderr,"Registering managed cluster with cluster id=%d\n", g_clusterid);
  if(check_cluster_exists())
    {
      fprintf(stderr,"Managed cluster is already registered -  cluster id=%d\n", g_clusterid);
      //      log_message_force(LOG_INFO, "Managed cluster is registered.",1);
      return 1;
    }
  if(add_cluster())
    {
      fprintf(stderr,"Managed cluster has been registered - registered cluster id=%d\n",g_clusterid);
      //      log_message_force(LOG_INFO, "Managed cluster has been registered.",1);
      return 1;
    }
  
  return -1;
}


int main(int argc, char** argv)
{
	ndb_init() ;     
	pthread_t monitor_thread;
	pthread_t le_thread;
	pthread_t maint_thread;
	//  pthread_t mysql_mon_thread;
	pthread_t mysql_coll_thread;
	pthread_t ndb_coll_thread;
	
	/** init defaults*/
	strcpy(params.db,"cmon");
	strcpy(params.user,"cmon");
	strcpy(params.passwd,"");
	strcpy(params.host,"127.0.0.1");
	params.port=3306;
	strcpy(params.connectstring,"127.0.0.1");
	strcpy(g_running_dir,"/tmp/");
	g_clusterid=1;
	int save_clusterlog_hours = 48;
	params.save_clusterlog_hours = save_clusterlog_hours;
	//  params.save_statistics_hours = save_statistics_hours;
	option(argc,argv);
	strncpy(cluster_conn[MONITOR_THREAD].connectstring,params.connectstring,strlen(params.connectstring));
	strncpy(cluster_conn[LOG_THREAD].connectstring,params.connectstring,strlen(params.connectstring));
	
	char msg[1024];
	sprintf(msg,
			"Starting cmon version %s with the following parameters:\n "
			//	      " --mysqldatabase=%s "
			"\t--mysqlpasswd=%s\n" 
			"\t--mysqlhost=%s\n" 
			"\t--mysqlport=%d\n" 
			"\t--ndb-connectstring=%s\n"
			"\t--savetime-clusterlog=%d (hours)\n",
			VERSION,
			//	      params.db,
			params.passwd,
			params.host,
			params.port,
			params.connectstring,
			params.save_clusterlog_hours
			);
	
	fprintf(stderr, "%s\n", msg);
	fprintf(stderr, "If that doesn't look correct, kill cmon and restart with -? for help on the parameters, or change the params in /etc/init.d/cmon\n\n");
	
	char* hostname=(char*)malloc(256 * sizeof(char));
	memset(hostname,0,256);
	
	get_hostname(hostname);
	fprintf(stderr, "You need to GRANT (and specify a password if you wish) the following on mysql on %s:\n\n",params.host);
	fprintf(stderr, "GRANT create,select,update,insert,delete on cmon.* to 'cmon'@'%s';\n",hostname);
	fprintf(stderr, "GRANT super on *.* to 'cmon'@'%s';\n\n",hostname);
	
	free(hostname);
	
	
	fprintf(stderr, "Testing connection to mysqld..\n");
	if(!test_connect())
    { 
		fprintf(stderr, "Testing connection failed\n");
		fprintf(stderr, "Terminating..\n");
		exit(-1);
    }
	fprintf(stderr, "Connection ok..\n");
	fprintf(stderr, "Please wait while cmon is starting up..\n");
	
	
	init_logger(g_log, g_nodaemon, g_logfilename);
	
	if(g_log)
    {
		FILE * logfile; 
		logfile=fopen(g_logfilename,"a");	  
		if(!logfile)
		{
			fprintf(stderr,"Cannot open %s\n", g_logfilename);
			exit(-1);
		}
		fclose(logfile);
    }
	
	
	while(1)
    {
		
		
		/*
		 * Two threads are created:
		 * - one to monitor the cluster state (monitorThread)
		 * - one to monitor log events (log_event_thread)
		 */
		
		
		if ( !checkSchema())
		{
			fprintf(stderr,"Recreating missing tables\n");
			createSchema(false);
			fprintf(stderr,"Done\n");
		}
		else
			fprintf(stderr,"Schema looks complete.\n");
		
		
		if(register_cmon() < 0)
		{
			fprintf(stderr,"cmon failed to register managed cluster.\n");
			//	  log_message_force(LOG_ERR,"cmon failed to register managed cluster.",1);
			cmon_exit(-1);
		}
		
		fprintf(stderr,"cmon has started successfully.\n");
		if(!g_nodaemon)
		{
			if(strcmp(g_logfilename,""))
				fprintf(stderr, "Going to daemoinze.. - cmon will write a log in %s from now\n",g_logfilename);
			else
				fprintf(stderr, "Going to daemoinze.. - cmon will write a log in syslog from now\n");
		}
		if(!g_nodaemon)
			daemonize();
		
		connect_ndb();      
		
		log_message(LOG_INFO,"Setting up threads to handle monitoring");
		
		pthread_create(&monitor_thread, 0, monitorThread, &params);
		sleep(5);
		pthread_create(&le_thread, 0, log_event_thread, &params);
		pthread_create(&maint_thread, 0,  cmon_db_maint_thread, &params);
		pthread_create(&mysql_coll_thread, 0, mysql_collector_thread, &params);
		//      pthread_create(&mysql_mon_thread, 0, mysql_monitor_thread, &params);
		pthread_create(&ndb_coll_thread, 0,  ndb_collector_thread, &params);
		
		
		bool running=true;
		while(running)
		{
			pthread_join(monitor_thread, 0);
			pthread_join(le_thread, 0);
			pthread_join(maint_thread, 0);
			//	  pthread_join(mysql_mon_thread, 0);
			pthread_join(mysql_coll_thread, 0);
			pthread_join(ndb_coll_thread, 0);
			running=false;
		}
		
		ndb_end(0);
		return 0;
    }
	
}
