/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "log.h"
#include "connect.h"

extern int g_clusterid;

void disconnectMySQL(mysql_t  * mysql)
{
  if(&mysql->mysql)
    mysql_close(&mysql->mysql);
}

bool connectMySQL(mysql_t * mysql)
{
  int max_retries=10;
  int retry=0;
  int retry_time=0;
  char msg[255];
  while(true)
    {
      mysql_init(&mysql->mysql);
      if (!&mysql->mysql)
	return false;

      if(!mysql_real_connect(&mysql->mysql, 
			     params.host, 
			     params.user, 
			     params.passwd, 
			     params.db, 
			     params.port, 
			     "",
			     0))
	{
	  retry++;	 
	  if(retry < 10)
	    retry_time=retry;
	  else
	    retry_time=10;
 
	  memset(msg,0,sizeof(msg));
	  snprintf(msg,sizeof(msg), 
		  "Failed to connect to database %s "
		  "user=%s, password=%s, host=%s, port=%d."
		  "Retrying in %d seconds"
		  ": Error: %s",
		  params.db,
		  params.user,
		  params.passwd,
		  params.host,
		  params.port,
		  retry_time,
		  mysql_error(&mysql->mysql));
	  log_message_force(LOG_WARNING, msg,1);
	  if(&mysql->mysql!=0)
	    mysql_close(&mysql->mysql);
	  sleep(retry_time);
	  continue;
	}
      
      //      log_message(LOG_INFO, "connected to mysqld");
      
      if(!executeQuery("SET SQL_LOG_BIN=0", mysql))
	{
	  char *hname = (char*)malloc(255*sizeof(char));
	  memset(hname,0,sizeof(hname));
	  get_hostname(hname);
	  fprintf(stderr, "cmon user must have SUPER privilege!\n");
	  fprintf(stderr, "GRANT super on *.* to 'cmon'@'%s';\n\n", hname);
	  free(hname);
	  return false;
	}
      
      return true;
    }
  memset(msg,0,sizeof(msg));
  snprintf(msg,sizeof(msg), "Failed to connect to database after %d retries: Error: %s.",
	  max_retries,
	  mysql_error(&mysql->mysql));
  log_message_force(LOG_WARNING, msg,1);
  log_message(LOG_INFO, "..exitting");
  return false; 
}



bool connectMySQL2(MYSQL * mysql, const char * db )
{
  int max_retries=10;
  int retry=0;
  int retry_time=0;
  char msg[255];
  while(true)
    {
      mysql_init(mysql);
      if (!mysql)
	return false;

      if(!mysql_real_connect(mysql, 
			     params.host, 
			     params.user, 
			     params.passwd, 
			     db,
			     params.port, 
			     0,
			     0))
	{
	  retry++;	  
	  
	  if(retry <= 10)
	    retry_time=retry;
	  else
	    break;
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg,sizeof(msg), 
		  "Failed to connect to database retrying in %d seconds"
		  ": Error: %s",
		  retry_time,
		  mysql_error(mysql));
	  log_message_force(LOG_WARNING, msg,1);
	  if(mysql!=0)
	    mysql_close(mysql);
	  sleep(retry_time);
	  continue;
	}           
      //      log_message(LOG_INFO, "connected to mysqld");
      if(!executeQuery2("SET SQL_LOG_BIN=0", mysql))
	{
	  char *hname = (char*)malloc(255*sizeof(char));
	  memset(hname,0,sizeof(hname));
	  get_hostname(hname);
	  fprintf(stderr, "cmon user must have SUPER privilege!\n");
	  fprintf(stderr, "GRANT super on *.* to 'cmon'@'%s';\n\n", hname);
	  free(hname);
	  return false;
	}
      return true;
    }
  memset(msg,0,sizeof(msg));
  snprintf(msg,sizeof(msg), "Failed to connect to database after %d retries: Error: %s",
	  max_retries,
	  mysql_error(mysql));
  log_message_force(LOG_WARNING, msg,1);
  log_message(LOG_INFO, "..exitting");
  return false; 
}


void get_hostname(char * hostname)
{
      if(strcmp("127.0.0.1",params.host)!=0)
	{
	  gethostname(hostname,sizeof(hostname));
	}
      else
	{
	  memcpy(hostname,params.host, sizeof(params.host));
	}
      hostname[strlen(params.host)]=0;
}

bool connectMySQL3(MYSQL * mysql, 
		   const char * hostname,
		   const char * username,
		   const char * password,
		   int port
		   )
{


  char msg[255];
  
  mysql_init(mysql);
  if (!mysql)
    return false;
  
  if(!mysql_real_connect(mysql, 
			 hostname, 
			 username, 
			 password, 
			 "",
			 port, 
			 0,
			 0))
    {
      memset(msg,0,sizeof(msg));
      snprintf(msg,sizeof(msg), 
	      "(stats collection): Failed to connect to mysql server on %s:%d "
	      ": Error: %s",
	      hostname,
	      port,
	      mysql_error(mysql));
      log_message(LOG_WARNING, msg);
       if(mysql!=0)
	  mysql_close(mysql);
      return false;
    } 
  if(!executeQuery2("SET SQL_LOG_BIN=0", mysql))
    {
      char *hname = (char*)malloc(255*sizeof(char));
      memset(hname,0,sizeof(hname));
      get_hostname(hname);
      fprintf(stderr, "cmon user must have SUPER privilege!\n");
      fprintf(stderr, "GRANT super on *.* to 'cmon'@'%s';\n\n", hname);
      free(hname);
      return false;
    }
  
  return true;
}


bool connectMySQL4(MYSQL * mysql, 
		   const char * hostname,
		   const char * username,
		   const char * password,
		   const char * socket,
		   int port
		   )
{


  char msg[255];
  
  mysql_init(mysql);
  if (!mysql)
    return false;
  
  if(!mysql_real_connect(mysql, 
			 0,
			 username, 
			 password, 
			 "" /*db*/,
			 0, 
			 0,
			 0))
    {
      memset(msg,0,sizeof(msg));
      snprintf(msg,sizeof(msg), 
	      "(stats collection): Failed to connect to mysql server on %s:%d "
	      ": Error: %s",
	      hostname,
	      port,
	      mysql_error(mysql));
      log_message(LOG_WARNING, msg);
       if(mysql!=0)
	  mysql_close(mysql);
      return false;
    }  
  return true;
}


bool test_connect()
{

  MYSQL mysql;
  int max_retries=3;
  int retry=0;
  int retry_time=0;
  char msg[255];
  //  while(true)
  // {
      mysql_init(&mysql);
      if(!mysql_real_connect(&mysql, 
			     params.host, 
			     params.user, 
			     params.passwd, 
			     "", 
			     params.port, 
			     0,
			     0))
	{
	  retry++;	 
	  if(retry < max_retries)
	    retry_time=retry;
	  else
	    {
	      return false;
	      retry_time=10;
	    }
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg,sizeof(msg), 
		   "\tERROR: Failed to connect to database %s with the following parameters:"
		   "user=%s, password=%s, host=%s, port=%d"
		   " :\n\tERROR: %s",
		   params.db,
		   params.user,
		   params.passwd,
		   params.host,
		   params.port,
		   mysql_error(&mysql));
	  fprintf(stderr, "%s\n", msg);
	  log_message_force(LOG_WARNING, msg,1);
	  mysql_close(&mysql);
	  return false;
	}
      //}
      mysql_close(&mysql);
      return true;
  
}


bool executeQuery(const char * query , mysql_t * mysql)
{
  char msg[2048];
  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      mysql_close(&mysql->mysql);
      connectMySQL(mysql);      
      if(mysql_ping(&mysql->mysql))
	{
	  return false;
	}
      return false;
    }
  return true;
}


bool executeQuery2(const char * query , MYSQL * mysql)
{
  char msg[2048];
  if(mysql_real_query(mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(mysql));
      log_message(LOG_WARNING, msg);
      mysql_close(mysql);
      if(mysql_ping(mysql))
	{
	  return false;
	}
      return false;
    }
  return true;
}


bool connectNdbMgm(cluster_conn_t * conn )
{
  char msg[1024];
  if(conn->handle!=0)
    {
      if(ndb_mgm_is_connected(conn->handle)>0)
	{
	  return true;
	}
    }
  
  if(conn->handle!=0)
    {
      ndb_mgm_destroy_handle(&conn->handle);
      conn->handle=0;
    }
  
  conn->handle = ndb_mgm_create_handle();
  if (ndb_mgm_set_connectstring(conn->handle, 
				conn->connectstring) == -1)
    {
      log_message(LOG_WARNING, "Could not set the connectstring for ndb_mgmd");
      return false;
    }
  if (ndb_mgm_connect(conn->handle,0,0,0) < 0)
    {
      snprintf(msg,sizeof(msg), "line=%d code: %d msg: %s", 
	      __LINE__,
	      ndb_mgm_get_latest_error(conn->handle), 
	      ndb_mgm_get_latest_error_msg(conn->handle));
      log_message(LOG_WARNING, msg);
      return false;
    }

  snprintf(msg,sizeof(msg), "connected to ndb_mgmd on host %s, port %d", 
	  ndb_mgm_get_connected_host(conn->handle),
	  ndb_mgm_get_connected_port(conn->handle));

  log_message(LOG_INFO, msg);
  return true;
}


int connect_ndb()
{
  if(g_ndb_conn)
    delete g_ndb_conn;
  
    retry:
      g_ndb_conn=new Ndb_cluster_connection(params.connectstring);      
      
      if(!g_ndb_conn)
	{
	  log_message(LOG_INFO,
		      "Allocation of Ndb_cluster_connection failed. Very bad. Do you have an empty [mysqld] slot in config.ini so that cmon can connect?");
	  cmon_exit(-1);
	}
      if(g_ndb_conn->connect(3,5,1)<0)
	{
	  log_message(LOG_INFO,"Could not connect to management server(s). - make sure you have an API slot free in config.ini (http:// www.severalnines.com/cmon/doc/apislot.html ). cmon will continue to work but the aggregation of ndb statistics such as diskdata info will not work.");
	  delete g_ndb_conn;
	  sleep(10);
	  goto retry;
	}
      
      if(g_ndb_conn->wait_until_ready(30,0)<0)
	{
	  log_message(LOG_INFO,"Could not connect to the data nodes. They are problably not started. Will retry in 10 seconds.");
	  delete g_ndb_conn;
	  sleep(10);
	  goto retry;
	}
      return 1;  
}


bool is_mysqld(const unsigned int nodeid, mysql_t * mysql)
{
  char msg[512];
  char query[128];
  snprintf(query,sizeof(query),"SELECT nodeid FROM mysql_server WHERE cid=%d AND nodeid=%d",g_clusterid, nodeid);
  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      
      return false;
    }
  else // query succeeded, process any data returned by it
    {
      MYSQL_RES * result = mysql_store_result(&mysql->mysql);
      if (result)  // there are rows
	{
	  
	  MYSQL_ROW row;

	  unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	  if(num_rows>0)
	    {
	      unsigned int n=0;
	      while ((row = mysql_fetch_row(result)))
		{
		  n=atoi(row[0]);
		}
	      if(n==nodeid)
		{
		  mysql_free_result(result);	 
		  return true;
		}
	      else 
		{
		  mysql_free_result(result);	 
		  return false;
		}
	    }	  
	  mysql_free_result(result);	 
	}
    }
  return false;
}


bool db_get_ndbd_hostname(const int nodeid,  char * hostname,  mysql_t * mysql)
{
  char msg[512];
  char query[128];
  snprintf(query,sizeof(query),"SELECT host FROM node_state WHERE cid=%d AND nodeid=%d",g_clusterid, nodeid);
  
  if(nodeid==0)
    return false;

  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      
      return false;
    }
  else // query succeeded, process any data returned by it
    {
      MYSQL_RES * result = mysql_store_result(&mysql->mysql);
      if (result)  // there are rows
	{
	  
	  MYSQL_ROW row;
	  unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	  if(num_rows>0)
	    {
	      while ((row = mysql_fetch_row(result)))
		{
		  if(row[0])
		    {
		      memcpy(hostname,row[0],strlen(row[0]));
		      hostname[strlen(row[0])]=0;		      
		    }
		}       
	    }	  
	  mysql_free_result(result);	 
	}
    }
  if(strcmp(hostname,"")!=0)
    return true;
  else return false;
}



bool db_get_ndbd_state(const int nodeid,  char * status,  mysql_t * mysql)
{
  char msg[512];
  char query[128];
  snprintf(query,sizeof(query),"SELECT status FROM node_state WHERE cid=%d AND nodeid=%d",g_clusterid, nodeid);

  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      
      return false;
    }
  else // query succeeded, process any data returned by it
    {
      MYSQL_RES * result = mysql_store_result(&mysql->mysql);
      if (result)  // there are rows
	{
	  
	  MYSQL_ROW row;
	  unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	  if(num_rows>0)
	    {
	      while ((row = mysql_fetch_row(result)))
		{
		  if(row[0])
		    {
		      memcpy(status,row[0],strlen(row[0]));
		    }
		}       
	    }	  
	  mysql_free_result(result);	 
	}
    }
  if(strcmp(status,"")!=0)
    return true;
  else return false;
}







bool db_check_node_exists(const int nodeid,   mysql_t * mysql)
{
  char msg[512];
  char query[128];
  snprintf(query,sizeof(query),"SELECT host FROM node_state WHERE cid=%d AND nodeid=%d",g_clusterid, nodeid);

  if(mysql_real_query(&mysql->mysql, query, strlen(query)))
    {
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s\n",
	      query,
	      mysql_error(&mysql->mysql));
      log_message(LOG_WARNING, msg);
      
      return false;
    }
  else // query succeeded, process any data returned by it
    {
      MYSQL_RES * result = mysql_store_result(&mysql->mysql);
      if (result)  // there are rows
	{
	  
	  unsigned long long num_rows = mysql_affected_rows(&mysql->mysql);
	  if(num_rows>0)
	    {
	      mysql_free_result(result);	 
	      return true;
	    }	  
	  mysql_free_result(result);	 
	}
    }
  return false;
}




void db_write_ndbd_nodestate(const int nodeid, 
			     const int node_group, 
			     const int version,
			     const char * status,  
			     const char * connect_addr,  
			     mysql_t * mysql)
{
  char buf[255];
  char hostname[128];
  if(connect_addr==0)
    {
      
      bool ret= db_get_ndbd_hostname(nodeid,  hostname,  &mysql_conn[MONITOR_THREAD]);
      if(ret)
	{
	  sprintf(buf,
		  "UPDATE node_state SET status='%s', report_ts=now() WHERE cid=%d AND nodeid=%d",	      
		  status,	    
		  g_clusterid, 
		  nodeid); 
	}

      else
	{
	  if(db_check_node_exists(nodeid,  &mysql_conn[MONITOR_THREAD]))
	    {
	      sprintf(buf,
		      "UPDATE node_state SET status='%s', report_ts=now() WHERE cid=%d AND nodeid=%d",	      
		      status,	    
		      g_clusterid, 
		      nodeid); 

	    }
	  else
	      sprintf(buf,
		      "insert ignore into node_state(cid,nodeid, nodegroup,status, node_type, version,report_ts) "
		      "values(%d,%d, %d,'%s', 'NDBD', '%d.%d.%d', now())",
		      g_clusterid, 
		      nodeid, 
		      node_group,
		      status, 		    
		      version >> 16 & 0xff,
		      version >> 8 & 0xff,
		      version >> 0 & 0xff);	    	     	      
	    
	}
    }
  else
    {
      if(db_check_node_exists(nodeid,  &mysql_conn[MONITOR_THREAD]))
	sprintf(buf,
		"UPDATE node_state SET status='%s' , report_ts=now() WHERE cid=%d AND nodeid=%d",	      
		status,	    
		g_clusterid, 
		nodeid); 
      else
	sprintf(buf,
		"insert ignore into node_state(cid,nodeid, nodegroup,status, node_type, version,host,report_ts) "
		"values(%d,%d,%d,'%s', 'NDBD', '%d.%d.%d', '%s', now())",
		g_clusterid, 
		nodeid, 
		node_group,
		status, 		    
		version >> 16 & 0xff,
		version >> 8 & 0xff,
		version >> 0 & 0xff,
		connect_addr      );	    	     
      
    }
  executeQuery(buf,&mysql_conn[MONITOR_THREAD]);	   

  
}

