#include "log.h"
#include "logger.h"
//#include "handler.h"

static  int logno=1;
static  char prev_message[4096];

void init_logger(bool log, bool nodaemon, char * logfilename)
{
  g_log=log;
  g_nodaemon=nodaemon;
  strcpy(g_logfilename,logfilename);
  pthread_mutex_init(&mut_log, NULL);
}


void log_message(int loglevel, const char *message)
{
  log_message_force(loglevel, message, 0);
}

void log_message_force(int loglevel, const char *message, int force)
{  
  pthread_mutex_lock(&mut_log); 


  char       buf[80];
  char cloglevel[16];	  	  
  if(force || strcmp(prev_message,message)!=0)
    {
      if(g_nodaemon || g_log)
	{
	  struct tm  ts;

	  time_t now;
	  time(&now);
	  ts = *localtime(&now);
	  strftime(buf, sizeof(buf), "%b %d %H:%M:%S", &ts);

	  switch (loglevel)
	    {
	    case LOG_INFO:
	      sprintf(cloglevel,"%s","INFO");
	      break;
	    case LOG_WARNING:
	      sprintf(cloglevel,"%s","WARNING");
	      break;
	    case LOG_ERR:
	      sprintf(cloglevel,"%s","ERROR");
	      break;
	    case LOG_ALERT:
	      sprintf(cloglevel,"%s","ALERT");
	      break;
	    case LOG_CRIT:
	      sprintf(cloglevel,"%s","CRITICAL");
	      break;
	    default:
	      sprintf(cloglevel,"%s","UNKNOWN");
	      break;
	    };
	  
	  FILE *logfile=NULL;
	  if(g_log && g_logfilename!=0)
	    {	      
	      /**
	       * rotate log if logfile >1M
	       * save five log files..	       
	       */
	      struct stat results;
	      if (stat(g_logfilename, &results) == 0)
		{	       
		  if(results.st_size > 1024*1024 )
		    {		    
		      char newlogfilename_next[255];		    
		      char newlogfilename_prev[255];		    
		      char newlogfilename[255];		    		
		      for(int i=logno;i>1;i--)
			{
			  memset(newlogfilename_next,0,
				 sizeof(newlogfilename_next));
			  memset(newlogfilename_prev,0,
				 sizeof(newlogfilename_prev));
			  sprintf(newlogfilename_next,
				  "%s.%d",g_logfilename,i);     
			  sprintf(newlogfilename_prev,
				  "%s.%d",g_logfilename,i-1);     
			  printf("going to rename %s to %s i=%d\n",
				 newlogfilename_prev,
				 newlogfilename_next,
				 i);
			  if(rename(newlogfilename_prev, newlogfilename_next)<0)
			    {
			      exit(-1);
			    }
			}
		      sprintf(newlogfilename,
				  "%s.%d",g_logfilename,1);     
		      if(rename(g_logfilename, newlogfilename)<0)
			{
			  exit(-1);
			}
		      logno++;		    
		      if(logno==6)
			logno=1;
		    }
		  logfile=fopen(g_logfilename,"a");	  
		  if(!logfile)
		    {
		      fprintf(stderr,"Cannot open %s\n", g_logfilename);
		      pthread_mutex_unlock(&mut_log); 
		      exit(-1);
		    }
		  fprintf(logfile,"%s : (%s) %s\n",buf, cloglevel , message);
		  fclose(logfile);
		  
		  
		}
	      else		
		exit(-1);			    
	    }
	  
	  if (g_nodaemon)	
	    {
	      fprintf(stderr,"%s : (%s) %s\n",buf, cloglevel , message);
	    }
	}
      else
	{
	  openlog ("cmon",LOG_PID,LOG_USER);          
	  //	  syslog (LOG_MAKEPRI(LOG_USER, loglevel),"%s", message);      
	  syslog (LOG_USER|loglevel,"%s", message);      
	  closelog ();
	}
      
//#if HAVE_DLFCN_H 
#if 0
      handler_log_message(loglevel, message);
#endif
    }
  strcpy(prev_message,message);
  pthread_mutex_unlock(&mut_log); 
}
