/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "maint.h"
#include "log.h"
#include "connect.h"
#include "statistics.h"

void * cmon_db_maint_thread(void * t)
{
  configparam_t * params = (configparam_t*)t;
  log_message(LOG_INFO, "Starting MaintenanceThread");    
    
  char queryCleanClusterLog[255];
  char queryCleanBackupLog[255];
  char queryCleanRestoreLog[255];
  char queryCleanBackup[255];
  char queryCleanRestore[255];

  int sleepTime=0;  
  
  while(1)
    {            
      sleep(5);
      sleepTime+=5;
      if(!connectMySQL(&mysql_conn[MAINT_THREAD]))
	{
	  usleep(1000*1000);
	  sleepTime=0;
	  continue;
	}

      		  
      if(sleepTime==3600)
	{
	  
	  sprintf(queryCleanClusterLog,
		  "DELETE FROM cluster_log "
		  "WHERE report_ts < subdate(NOW(), INTERVAL %d HOUR)",
	      params->save_clusterlog_hours);
	  executeQuery(queryCleanClusterLog, &mysql_conn[MAINT_THREAD]);

	  int r= selectCount("backup_log");
	  int limit=0;
	  if(r>100)
	    {
	      limit=r-100;
	      sprintf(queryCleanBackupLog,
		      "DELETE FROM backup_log ORDER BY report_ts LIMIT %d",
		      limit);
	      executeQuery(queryCleanBackupLog, &mysql_conn[MAINT_THREAD]);
	    }
	  r= selectCount("restore_log");	  
	  if(r>100)
	    {
	      limit=r-100;
	      sprintf(queryCleanRestoreLog,
		      "DELETE FROM restore_log ORDER BY report_ts LIMIT %d", 
		      limit);
	      executeQuery(queryCleanRestoreLog, &mysql_conn[MAINT_THREAD]);
	    }

	  r= selectCount("backup");
	  if(r>100)
	    {
	      limit=r-100;
	      sprintf(queryCleanBackup,
		      "DELETE FROM backup ORDER BY report_ts LIMIT %d",
		      limit);
	      executeQuery(queryCleanBackup, &mysql_conn[MAINT_THREAD]);
	    }
	  r= selectCount("restore");	  
	  if(r>100)
	    {
	      limit=r-100;
	      sprintf(queryCleanRestore,
		      "DELETE FROM restore ORDER BY report_ts LIMIT %d", 
		      limit);
	      executeQuery(queryCleanRestore, &mysql_conn[MAINT_THREAD]);
	    }
	  sleepTime=0;
	}
      disconnectMySQL(&mysql_conn[MAINT_THREAD]);
    }
   return 0;
}


int selectCount(const char * table)
{
  
  int records=0;
  char sqlbuf[512], msg[512];
  MYSQL mysql = mysql_conn[MAINT_THREAD].mysql;
  memset(sqlbuf,0,sizeof(sqlbuf));
  sprintf( sqlbuf,
	   "select count(*) from %s", table);
  
  int a=strlen(sqlbuf);
  if(mysql_real_query( &mysql, sqlbuf, a ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      sqlbuf,
	      mysql_error(&mysql));
      log_message(LOG_ERR, msg);
	  cmon_exit(mysql_errno(&mysql));
    }
  
  else // query succeeded, process any data returned by it
    {
      MYSQL_RES *result;
      result = mysql_store_result(&mysql);
      if (result)  // there are rows
	{
	  unsigned long long num_rows = mysql_affected_rows(&mysql);
	  if(num_rows==0)
	    {
	      mysql_free_result(result);
	      return 0;
	    }
	  else
	    {
	      
	      MYSQL_ROW row;
	      unsigned int num_fields;
	      unsigned int i;
	      
	      num_fields = mysql_num_fields(result);
	      while ((row = mysql_fetch_row(result)))
		{
		  unsigned long *lengths;
		  lengths = mysql_fetch_lengths(result);
		  for(i = 0; i < num_fields; i++)
		    {
		      records=atoi(row[i]);
		      mysql_free_result(result);
		      return records;
		    }		
		}
	      
	    }
	}
      else
	{
	  return 0;
	}
    }
  return 0;
}

