/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#include "connect.h"
#include "log.h"
#include "alarm.h"

#include "handler.h"
#include <mgmapi_debug.h>


extern int g_plugin_count;
extern int g_clusterid;

bool startNode(int nodeid)
{  
  int ret=0;
  if( cluster_state.block_restart )
    {
      char msg[128];
      sprintf(msg, "MonitorThread: REFUSED TO START NODE %d SINCE CLUSTER HAS FAILED - MANUAL INTERVENTION NEEDED\n", nodeid);
      log_message(LOG_WARNING,msg);    
      return false;
    }
  
  if(cluster_state.nodes[nodeid].startmode == CS_INR)
    {
      ret = ndb_mgm_restart2(cluster_conn[MONITOR_THREAD].handle,
			     1,
			     &nodeid, 1 /**--initial*/, 
			     1 /*no started*/,1 /*force*/);
      ret= ndb_mgm_start(cluster_conn[MONITOR_THREAD].handle,
			 1,
			 &nodeid);
    }
  else    
    {
	ret= ndb_mgm_start(cluster_conn[MONITOR_THREAD].handle,
			   1,
			   &nodeid);
    }
  if (ret == -1)
    {
      char msg[128];
      sprintf(msg, "MonitorThread: FAILED TO START NODE nodeid = %d\n", nodeid);
      log_message(LOG_WARNING,msg);
      return false;
    }
  return true;

}


bool startCluster()
{
  if( cluster_state.block_restart )
    {
      char msg[128];
      sprintf(msg, "MonitorThread: REFUSED TO START CLUSTER SINCE IT FAILED - MANUAL INTERVENTION NEEDED\n");
      log_message(LOG_WARNING,msg);    
      return false;
    }
  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_UNKNOWN)
	continue;
      
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_NDB)
	{
	  if(cluster_state.nodes[i].status==NDB_MGM_NODE_STATUS_NOT_STARTED)
	    {
	      int ret= ndb_mgm_start(cluster_conn[MONITOR_THREAD].handle,
				     1,
				     &i);
	      if (ret == -1)
		{
		  char msg[128];
		  sprintf(msg,"MonitorThread: FAILED TO START NODE nodeid = %d", i);
		  log_message(LOG_WARNING,msg);
		  return false;
		}
	      
	    }	    
	}
    }
  return true;
}


/*
 * If one node from each node group then cluster is started
 * but we don't check node groups yet..
 * and it is not really necessary, since if a node group
 * is dead, then we have no cluster..
 */
bool clusterStarted()
{
  cluster_state.previous_state=cluster_state.current_state;
  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_UNKNOWN)
	{
	  continue;
	}
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_NDB)
	{
	  if(cluster_state.nodes[i].status==NDB_MGM_NODE_STATUS_STARTED)
	    {	   	      

	      return true;
	    }
	}
    }
  return false;
}


/*
 * Check if all nodes are started
 */
bool clusterAllNdbStarted()
{

  cluster_state.previous_state=cluster_state.current_state;

  bool started=true;
  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type != NDB_MGM_NODE_TYPE_NDB)
	continue;
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_NDB)
	{
	  if(cluster_state.nodes[i].status==NDB_MGM_NODE_STATUS_STARTED)
	    {
	      started=true;
	    }
	  else
	    { 
	      /*
	      char buf[128];
	      sprintf(buf,"%s","cluster STARTED" 
	      "   - but running in degraded state");
	      if(cluster_state.current_state!=CS_RECOVERING)
		cluster_state.current_state=CS_DEGRADED;
	      logger (buf);
	      */
	      return false;
	    }
	}
    }
  if(started)
    {
      char buf[128];
      cluster_state.current_state=CS_STARTED;
      cluster_state.block_restart=false; 
      sprintf(buf,"%s","MonitorThread: Cluster STARTED" 
	      "   - all nodes are started");
      log_message (LOG_INFO,buf);

    }
  return started;
}



bool clusterStarting()
{
  bool starting=false;
  cluster_state.previous_state=cluster_state.current_state;
  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_UNKNOWN)
	continue;
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_NDB)
	{
	  if(cluster_state.nodes[i].status==NDB_MGM_NODE_STATUS_STARTING)
	    {
	      starting = starting | true;
	    }
	}
    }
  if(starting)
    {
      cluster_state.current_state=CS_STARTING;
      log_message(LOG_INFO,"MonitorThread: Cluster STARTING");
    }
  return starting;
}


bool clusterNotStarted()
{
  bool not_started=true;
  cluster_state.previous_state=cluster_state.current_state;
  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type != NDB_MGM_NODE_TYPE_NDB)
	{
	  continue;
	}
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_NDB)
	{
	  if(cluster_state.nodes[i].status!=NDB_MGM_NODE_STATUS_NOT_STARTED)
	    {
	      return false;
	    }
	}
    }
  if(not_started)
    {
      cluster_state.current_state=CS_NOT_STARTED;
      log_message(LOG_INFO,"MonitorThread: Cluster NOT_STARTED");
    }
  return not_started;
}




bool clusterShuttingDown()
{
  bool stopping=false;
  cluster_state.previous_state=cluster_state.current_state;
  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_UNKNOWN)
	continue;

      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_NDB)
	{
	  if(cluster_state.nodes[i].status==NDB_MGM_NODE_STATUS_SHUTTING_DOWN)
	    {
	      stopping = stopping | true;
	    }
	}
    }
  if(stopping)
    {
      cluster_state.current_state= CS_SHUTTING_DOWN;
      log_message(LOG_WARNING,"MonitorThread: Cluster SHUTTING DOWN");
    }

  return stopping;
}



bool clusterFailure()
{
  bool failure=false;
  cluster_state.previous_state=cluster_state.current_state;
  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_UNKNOWN)
	continue;


      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_NDB)
	{
	  if(cluster_state.nodes[i].status==NDB_MGM_NODE_STATUS_NO_CONTACT)
	    {
	      failure = true;
	    } 
	  else
	    {
	      return false;
	    }
	}
    }
  if(failure  && cluster_state.previous_state!=CS_FAILURE)
    {
      db_write_alarm(&mysql_conn[MONITOR_THREAD],			   
		     g_clusterid,
		     0,
		     CS_FAILURE,
		     cluster_state.current_state==CS_DEGRADED ? CS_DEGRADED: cluster_state.previous_state  ,			  
		     "ALERT",
		     "CLUSTER",
		     "CLUSTER FAILURE",
		     "",
		     "The cluster has failed - inspect error logs",
		     "Try to restart the data nodes - file a bug report or contact MySQL Support");
      cluster_state.current_state=CS_FAILURE; 
      cluster_state.block_restart=true; 
      log_message(LOG_CRIT,
		  "MonitorThread: Cluster FAILURE - MANUAL INTERVENTION REQUIRED");
      printf("BBBBBBBBBBBBBBBBBLLLLLLLA %d\n\n",  
	     cluster_state.block_restart);
    }
  return failure;
}

void handleState()
{
  bool degraded =false;

  for(int i=cluster_state.minNdbNodeId; i<=cluster_state.maxNdbNodeId;i++)
    {
      if(cluster_state.nodes[i].type == NDB_MGM_NODE_TYPE_UNKNOWN)
	continue;	  	  
      
      if(cluster_state.nodes[i].type != NDB_MGM_NODE_TYPE_NDB)
	{
	  continue;
	}
      /*
       * if one reports NO_CONTACT or SHUTTING DOWN, then degraded
       */
      char sqlbuf[256];

      switch(cluster_state.nodes[i].status)
	{
	case NDB_MGM_NODE_STATUS_NO_CONTACT:
	case NDB_MGM_NODE_STATUS_SHUTTING_DOWN:
	  {
	    cluster_state.current_state=CS_DEGRADED;
	    char buf[128];
	    sprintf(buf, 
		    "MonitorThread: Cluster Started but in DEGRADED STATE"
		    "   - NO CONTACT WITH FAILED NODE ID %d", i);
	    log_message(LOG_WARNING,buf);
	    degraded=true;
	    memset(sqlbuf,0,sizeof(sqlbuf));
	    sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'DEGRADED')", g_clusterid);
	    executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	    break;
	  }
	case NDB_MGM_NODE_STATUS_NOT_STARTED:
	  {
	    cluster_state.current_state=CS_DEGRADED;
	    /*
	     * If Ok to start lets recover.
	     */
	    char buf[128];
	    sprintf(buf, 
		    "MonitorThread: Cluster Started but in DEGRADED STATE "
		    "   - NOT STARTED NODE ID %d",i);
	    log_message(LOG_WARNING,buf);
	    
	    
	    if(clusterStarted() && 
	       cluster_state.nodes[i].startIsOk)
	      {
		cluster_state.current_state=CS_DEGRADED;
		char buf[128];
		sprintf(buf, 
			"MonitorThread: Cluster Started but in DEGRADED STATE"
			"   - WILL ATTEMPT TO START NODE ID %d", i);
		log_message(LOG_WARNING,buf);
		startNode(i);
	      }
	    memset(sqlbuf,0,sizeof(sqlbuf));
	    sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'DEGRADED')", g_clusterid);
	    executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);

	    break;
	  }	  
	case NDB_MGM_NODE_STATUS_STARTING:
	  {
	    cluster_state.current_state=CS_RECOVERING;
	    char buf[128];
	    sprintf(buf, 
		    "MonitorThread: Cluster Started but in DEGRADED STATE"
		    "   -  RECOVERING NODE ID %d", i);
	    log_message(LOG_WARNING,buf);
	    memset(sqlbuf,0,sizeof(sqlbuf));
	    sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'RECOVERING')", g_clusterid);
	    executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	    
	    break;
	  }
	default:
	  break;
	};       
    }
}


void clusterState()
{
  char sqlbuf[256];
  if(clusterStarted())
    {
      if (clusterAllNdbStarted())
	{
	    memset(sqlbuf,0,sizeof(sqlbuf));
	    sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'STARTED')", g_clusterid);
	    executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	    db_cluster_clear_alarm( &mysql_conn[MONITOR_THREAD]);
	  return;
	}
      handleState();
    } 
  else
    {
      /*
       * We have no cluster!!
       */
      if(clusterFailure())
	{
	  cluster_state.block_restart=true; 
	  memset(sqlbuf,0,sizeof(sqlbuf));
	  sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'FAILURE')", g_clusterid);
	  executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	}
      if(clusterStarting())
	{
	  memset(sqlbuf,0,sizeof(sqlbuf));
	  sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'STARTING')", g_clusterid);
	  executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	}

      if(clusterNotStarted())
	{
	  /**
	   * if conditions are correct we will start the cluster
	   * (i.e., all nodes are NOT_STARTED
	   */
	  memset(sqlbuf,0,sizeof(sqlbuf));
	  sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'NOT_STARTED')", g_clusterid);
	  executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	  startCluster();
	}

      if(clusterShuttingDown())
	{
	  memset(sqlbuf,0,sizeof(sqlbuf));
	  sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'SHUTTING_DOWN')", g_clusterid);
	  executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	}
    }
}

/**
 * More to do here. There is more information that can be stored
 */
void setNodeStatus(ndb_mgm_node_state *state)
{
  cluster_state.nodes[state->node_id].prev_status=  
    cluster_state.nodes[state->node_id].status;
  cluster_state.nodes[state->node_id].status = state->node_status;
  cluster_state.nodes[state->node_id].type = state->node_type;
  
  char buf[255];
  if(state->node_type ==  NDB_MGM_NODE_TYPE_NDB)
    {
      
      switch(state->node_status)
	{
	case NDB_MGM_NODE_STATUS_UNKNOWN:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "UNKNOWN",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);
	    /*
	    sprintf(buf,
		    "replace into node_state(cid, nodeid,status,node_type)"
		    "values(%d,%d,'UNKNOWN','NDBD')",g_clusterid, state->node_id);	    
	    executeQuery(buf, &mysql_conn[MONITOR_THREAD]);
	    */
	    break;
	  }
	case NDB_MGM_NODE_STATUS_NO_CONTACT:
	  {

	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "NO_CONTACT",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);

	    cluster_state.current_state=CS_DEGRADED;


	    break;
	  }
	case NDB_MGM_NODE_STATUS_NOT_STARTED:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "NOT_STARTED",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);
	    /*
	    sprintf(buf,
		    "replace into node_state(cid,nodeid,status,node_type)"
		    "values(%d,%d,'NOT_STARTED','NDBD')",g_clusterid, state->node_id);	    
	    executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
	    */
	    break;
	  }
	case NDB_MGM_NODE_STATUS_STARTING:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "STARTING",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);
	    /*
	    sprintf(buf,
		    "replace into node_state(cid,nodeid,status,node_type,host, version)"
		    "values(%d,%d,'STARTING','NDBD',  '%s','%d.%d.%d')",g_clusterid, state->node_id, state->connect_address, 	 
		    state->version >> 16 & 0xff,
		    state->version >> 8 & 0xff,
		    state->version >> 0 & 0xff
		    );	    
	    executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
	    */
	    break;
	  }
	case NDB_MGM_NODE_STATUS_STARTED:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "STARTED",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);


	    db_node_clear_alarm( &mysql_conn[MONITOR_THREAD],		      
				 state->node_id,
				 "NDBD",
				 "ALERT",
				 "NODE DISCONNECTED");
	    
				 clusterAllNdbStarted();
	    break;
	  }
	case NDB_MGM_NODE_STATUS_SHUTTING_DOWN:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "SHUTTING_DOWN",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);
	    /*
	    sprintf(buf,
		    "replace into node_state(cid,nodeid,status,node_type,host,version)"
		    "values(%d,%d,'SHUTTING_DOWN','NDBD', '%s','%d.%d.%d')",g_clusterid, state->node_id, state->connect_address, 	 
		    state->version >> 16 & 0xff,
		    state->version >> 8 & 0xff,
		    state->version >> 0 & 0xff
		    );	    
	    executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
	    */
	    break;
	  }
	case NDB_MGM_NODE_STATUS_RESTARTING:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "RESTARTING",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);
	    /*
	    sprintf(buf,
		    "replace into node_state(cid,nodeid,status, node_type,host,version)"
		    "values(%d,%d,'RESTARTING','NDBD',  '%s','%d.%d.%d')",g_clusterid, state->node_id, state->connect_address, 	 
		    state->version >> 16 & 0xff,
		    state->version >> 8 & 0xff,
		    state->version >> 0 & 0xff
		    );	    	    
	    executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
	    */
	    break;
	  }
	case NDB_MGM_NODE_STATUS_SINGLEUSER:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "SINGLEUSER",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);
	    /*
	    sprintf(buf,
		    "replace into node_state(cid,nodeid,status,node_type,host,version)"
		    "values(%d,%d,'SINGLEUSER','NDBD',  '%s','%d.%d.%d')",g_clusterid, state->node_id, state->connect_address, 	 
		    state->version >> 16 & 0xff,
		    state->version >> 8 & 0xff,
		    state->version >> 0 & 0xff
		    );	    
	    executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
	    */
	    break;
	  }
	case NDB_MGM_NODE_STATUS_RESUME:
	  {
	    db_write_ndbd_nodestate(state->node_id, 
				    state->node_group, 
				    state->version,
				    "RESUME",
				    state->connect_address, 	   
				    &mysql_conn[MONITOR_THREAD]);
	    /*
	    sprintf(buf,
		    "replace into node_state(cid,nodeid,status,node_type,host,version)"
		    "values(%d,%d,'RESUME','NDBD',  '%s','%d.%d.%d')",g_clusterid, state->node_id, state->connect_address, 	 
		    state->version >> 16 & 0xff,
		    state->version >> 8 & 0xff,
		    state->version >> 0 & 0xff
		    );	    
	    executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
	    */
	    break;
	  }
	default:
	  break;
	}
    }

  if(state->node_type ==  NDB_MGM_NODE_TYPE_API)
    {
      bool clear_alarm=false;
      if(strcmp(state->connect_address,"0.0.0.0")==0)	
	sprintf(buf,
		"replace into node_state(cid,nodeid,status,node_type,host,disconnects)"
		"values(%d,%d,'NO_CONTACT','API','%s',%d)",
		g_clusterid, state->node_id,
		state->connect_address,state->connect_count);	    	    
      else
	{
	  sprintf(buf,
		  "replace into node_state(cid,nodeid,status,node_type,host,disconnects,version)"
		  "values(%d,%d,'CONNECTED','API','%s',%d,'%d.%d.%d')",
		  g_clusterid,state->node_id,
		  state->connect_address,state->connect_count,		    
		  state->version >> 16 & 0xff,
		  state->version >> 8 & 0xff,
		  state->version >> 0 & 0xff);	    
	  clear_alarm=true;
	}
      executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
      if(clear_alarm)
	{
	  if(is_mysqld(state->node_id,&mysql_conn[MONITOR_THREAD]))
	    {
	      db_node_clear_alarm( &mysql_conn[MONITOR_THREAD],		      
				   state->node_id,
				   "MYSQLD",
				   "ALERT",
				   "NODE DISCONNECTED");
	    }
	}
    }

  if(state->node_type ==  NDB_MGM_NODE_TYPE_MGM)
    {
      if(strcmp(state->connect_address,"0.0.0.0")==0)	
	sprintf(buf,
		"replace into node_state(cid,nodeid,status,node_type)"
		"values(%d,%d,'NO_CONTACT','NDB_MGMD')", g_clusterid,state->node_id);	    
      else
		sprintf(buf,
			"replace into node_state(cid,nodeid,status,node_type,host,version)"
			"values(%d,%d,'CONNECTED','NDB_MGMD','%s','%d.%d.%d')",g_clusterid, state->node_id,state->connect_address,	
			state->version >> 16 & 0xff,
			state->version >> 8 & 0xff,
			state->version >> 0 & 0xff);	    
      executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
    }
}




/*
 * The monitorThread will periodically check the 
 * status of the cluster from the management server.
 * See the clusterState() function.
 */
void *  monitorThread(void * t)
{
  log_message(LOG_INFO, "Starting MonitorThread");    
  
 retry:
  log_message(LOG_INFO, "Trying to determine cluster state");

  if(!connectMySQL(&mysql_conn[MONITOR_THREAD]))
    {
      exit(-1);
    }
  cluster_state.block_restart=false; 
  
  while(!connectNdbMgm(&cluster_conn[MONITOR_THREAD]))
    {
      usleep(1000*1000);
      char sqlbuf[256];
      memset(sqlbuf,0,sizeof(sqlbuf));
      sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'MGMD_NO_CONTACT')", g_clusterid);
      executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
    }
  cluster_state.minNdbNodeId=256;
  cluster_state.maxNdbNodeId=0;
  cluster_state.current_state=CS_UNKNOWN;
  
  sleep(1);
  
  struct ndb_mgm_cluster_state * state = 
    ndb_mgm_get_status(cluster_conn[MONITOR_THREAD].handle);
  if(state==0)
    goto retry;
  
  cluster_state.noOfNodes=state->no_of_nodes;

  for(int i=0; i < state->no_of_nodes; i++) 
    {
      struct ndb_mgm_node_state *node_state= &state->node_states[i];       
      if(node_state!=0)
	setNodeStatus(node_state);	      	    
    }   

  
  char msg[256];
  sprintf(msg, "MonitorThread: Number of Nodes = %d", state->no_of_nodes);
  log_message(LOG_INFO, msg);
  memset(msg,0,sizeof(msg));

  handleState();
  if(clusterFailure()){
    cluster_state.current_state=CS_FAILURE;
    cluster_state.block_restart=true;
  }

  
  for(int i=0; i < state->no_of_nodes; i++) 
    {
      struct ndb_mgm_node_state *node_state= &state->node_states[i];
      
      switch(node_state->node_type)
	{
	case NDB_MGM_NODE_TYPE_NDB:
	  {	    
	    //	    printf("MonitorThread: NDB node id = %d", node_state->node_id);
	    //	    createNdbNode(node_state);	      
	    
	    if( node_state->node_id < cluster_state.minNdbNodeId)
	      cluster_state.minNdbNodeId=node_state->node_id;
	    if(node_state->node_id > cluster_state.maxNdbNodeId)
		  cluster_state.maxNdbNodeId=node_state->node_id;
		

	    if (node_state->node_status== NDB_MGM_NODE_STATUS_NOT_STARTED)
	      {
		/**
		 * if we have a node that is NOT_STARTED when 
		 * the monitor starts, then start that node
		 */

  
		sprintf(msg, "MonitorThread: Starting NOT_STARTED ndbd node %d", 
		       node_state->node_id);	
		log_message(LOG_INFO, msg);
		memset(msg,0,sizeof(msg));
		startNode(node_state->node_id);	    
	      }

	    if (node_state->node_status== NDB_MGM_NODE_STATUS_NO_CONTACT)
	      {

		/**
		 * if we have a node that is NO_CONTACT (not connected) when 
		 * the monitor starts, then we have to tell the monitor
		 * that is is ok to start that node when it comes NOT_STARTED
		 */
		if(!cluster_state.block_restart)
		  cluster_state.nodes[node_state->node_id].startIsOk=true;
	      }
	    if (node_state->node_status== NDB_MGM_NODE_STATUS_UNKNOWN)
	      {
		
		sprintf(msg,"MonitorThread: Unknown status of ndbd node %d", 
		       node_state->node_id);	
		log_message(LOG_WARNING, msg);
		memset(msg,0,sizeof(msg));
	      }

	    if (node_state->node_status== NDB_MGM_NODE_STATUS_STARTING)
	      {		
		sprintf(msg,"MonitorThread: NDBD node %d is STARTING", 
		       node_state->node_id);	
		log_message(LOG_INFO, msg);
		memset(msg,0,sizeof(msg));
	      }

	    if (node_state->node_status== NDB_MGM_NODE_STATUS_STARTED)
	      {
		
		sprintf(msg,"MonitorThread: NDBD node %d is STARTED", 
		       node_state->node_id);	
		log_message(LOG_INFO, msg);
		memset(msg,0,sizeof(msg));
	      }
	    cluster_state.nodes[node_state->node_id].used_dm=0;
	    cluster_state.nodes[node_state->node_id].used_im=0;
	    cluster_state.nodes[node_state->node_id].total_dm=0;
	    cluster_state.nodes[node_state->node_id].total_im=0;
	    break;  
	    }	    

	case NDB_MGM_NODE_TYPE_MGM:
	  {
	    sprintf(msg,"MonitorThread: MGM node id = %d", node_state->node_id);
	    //	    createMgmNode(node_state);	      
	    log_message(LOG_INFO, msg);
	    memset(msg,0,sizeof(msg));
	    break;  
	    }	    

	case NDB_MGM_NODE_TYPE_API:
	  {
	    sprintf(msg,"MonitorThread: API node id = %d", node_state->node_id);
	    log_message(LOG_INFO, msg);
	    memset(msg,0,sizeof(msg));
	    //createApiNode(node_state);	      
	    break;  
	    }	    
	default:
	    break;
	};
    }
  if(state!=0)
    free(state);
  
  disconnectMySQL(&mysql_conn[MONITOR_THREAD]);
  while(true)
    {
      while(!connectNdbMgm(&cluster_conn[MONITOR_THREAD]))
	{
	  char sqlbuf[256];
	  memset(sqlbuf,0,sizeof(sqlbuf));
	  sprintf(sqlbuf,"replace into cluster_state(id,status) values(%d,'MGMD_NO_CONTACT')", g_clusterid);
	  executeQuery(sqlbuf, &mysql_conn[MONITOR_THREAD]);
	  usleep(1000*1000);
	}
      if(!connectMySQL(&mysql_conn[MONITOR_THREAD]))
	{
	  usleep(1000*1000);
	  continue;
	}
      if(cluster_conn[MONITOR_THREAD].handle==0)
	{
	  continue;
	}
      struct ndb_mgm_cluster_state *state = 
	ndb_mgm_get_status(cluster_conn[MONITOR_THREAD].handle);
      if(state==0)
	continue;
      char buf[256];
      sprintf(buf,"DELETE FROM node_state WHERE status='NO_CONTACT'");
      executeQuery(buf,&mysql_conn[MONITOR_THREAD]);
      for(int i=0; i < state->no_of_nodes; i++) 
	{
	  struct ndb_mgm_node_state *node_state= &state->node_states[i];       
	  if(node_state!=0)
	    setNodeStatus(node_state);	      	
	}   
      clusterState();    
      disconnectMySQL(&mysql_conn[MONITOR_THREAD]);
      free(state);
      sleep(5);
    }
  ndb_mgm_destroy_handle(&cluster_conn[MONITOR_THREAD].handle); 
  return 0;
}
