/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "mysql_coll.h"
#include "log.h"
#include "connect.h"
#include "statistics.h"

extern int g_clusterid;

void * mysql_collector_thread(void * t)
{

  char msg[1024];
  MYSQL * mysql;
  log_message(LOG_INFO, "Starting MySQLCollectorThread");    
  while(1)
    {      
      if(!connectMySQL(&mysql_conn[MYSQL_COLL_THREAD]))
	{
	  sleep(10);
	  continue;
	}  
      mysql=&mysql_conn[MYSQL_COLL_THREAD].mysql;
      char sqlbuf[255];
      memset(sqlbuf,0,sizeof(sqlbuf));
      snprintf( sqlbuf,sizeof(sqlbuf),"SELECT cid, id, nodeid, hostname, username, password, role,port FROM mysql_server where cid=%d", g_clusterid);
      int a=strlen(sqlbuf);
      if(mysql_real_query( mysql, sqlbuf, a ))
	{
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg, sizeof(msg), "Query %s failed: Error: %s",
                  sqlbuf,
		  mysql_error(mysql));
	  log_message(LOG_ERR, msg);
	  mysql_close(mysql);
	  sleep(10);
	  continue;
	}       
      else // query succeeded, process any data returned by it
	{
	  MYSQL_RES * result = mysql_store_result(mysql);
	  if (result)  // there are rows
	    {
	      MYSQL_ROW row;
	      unsigned int num_fields;
	      if(!executeQuery("DELETE FROM mysql_global_statistics", &mysql_conn[MYSQL_COLL_THREAD]))
		continue;
	      num_fields = mysql_num_fields(result);
	      while ((row = mysql_fetch_row(result)))
		{
		  unsigned int cid=atoi(row[0]);
		  unsigned int id=atoi(row[1]);
		  unsigned int nodeid=atoi(row[2]);
		  const char * hostname=row[3];
		  const char * username=row[4];
		  const char * password=row[5];
		  const char * role=row[6];
		  unsigned int port=atoi(row[7]);
		  int newnodeid= 0;
		  if(nodeid==0)
		    newnodeid= update_server_tbl(cid,id, hostname, username, password, port);
		  else
		    newnodeid=nodeid;
		  //		  insert_stats_tbl(cid,id, (unsigned int)newnodeid);		      
		  collect_mysql_stats(id, nodeid, hostname, username ,password, role,port);
		  collect_mysql_slave_stats(id,hostname, username, password, port,role );
		  collect_mysql_master_stats(id,hostname, username, password, port,role );
		  //collect_mysql_variables(atoi(row[0]), row[2], row[3],row[4], row[5],atoi(row[6]);

		}	       
	      mysql_free_result(result);
	    }
	}
      mysql_close(mysql);	       
      sleep(10);       
    }
  return 0;
}


int update_server_tbl(const unsigned int cid,const unsigned int id, const  char * hostname, 
		      const char * username, const char * password, const unsigned int port)
{
  char sql[1024];
  


  char msg[1024];
  MYSQL   mysql;
  bool ok=false;
  int nodeid=0;
	      
  ok=connectMySQL3(&mysql, hostname, username, password,  port);

  if(ok)
    {
      
      char sqlbuf[255];
      memset(sqlbuf,0,sizeof(sqlbuf));

      snprintf( sqlbuf,sizeof(sqlbuf),"select VARIABLE_VALUE from information_schema.global_status WHERE VARIABLE_NAME='Ndb_cluster_node_id'");

      int a=strlen(sqlbuf);
      if(mysql_real_query( &mysql, sqlbuf, a ))
	{
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg, sizeof(msg),"Query %s failed: Error: %s",
                  sqlbuf,
		  mysql_error(&mysql));
	  log_message(LOG_ERR, msg);
	  mysql_close(&mysql);
	  return 0;
	}
      
      else // query succeeded, process any data returned by it
	{
	  MYSQL_RES * result = mysql_store_result(&mysql);
	  if (result)  // there are rows
	    {
	      
	      MYSQL_ROW row;
	      while ((row = mysql_fetch_row(result)))
		{
		  nodeid= atoi(row[0]);
		}		  
	      if(nodeid>0)
		{
		  snprintf(sql,sizeof(sql),"UPDATE mysql_server SET connected=1, nodeid=%d WHERE cid=%d and id=%d", nodeid, cid,id);
		  //		  printf("SQL=%s\n", sql);
		  if(!executeQuery(sql,&mysql_conn[MYSQL_COLL_THREAD]))
		    {
		      return 0;
		    }	
		}
	    }
	}
    }
  
  mysql_close(&mysql);
  
  return nodeid;
}



void update_server_tbl_set_role(const unsigned int cid,const unsigned int id, const  char * hostname, 
			   const char * username, const char * password, const unsigned int port, const char * role)
{
  char sql[1024];
  


	      
  /*
  ok=connectMySQL3(&mysql, hostname, username, password,  port);

  if(ok)
    {
      
      char sqlbuf[255];
      memset(sqlbuf,0,sizeof(sqlbuf));

      snprintf( sqlbuf,sizeof(sqlbuf),"select VARIABLE_VALUE from information_schema.global_status WHERE VARIABLE_NAME='Ndb_cluster_node_id'");

      int a=strlen(sqlbuf);
      if(mysql_real_query( &mysql, sqlbuf, a ))
	{
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg, sizeof(msg),"Query %s failed: Error: %s",
                  sqlbuf,
		  mysql_error(&mysql));
	  log_message(LOG_ERR, msg);
	  mysql_close(&mysql);
	}
      
      else // query succeeded, process any data returned by it
	{
	  MYSQL_RES * result = mysql_store_result(&mysql);
	  if (result)  // there are rows
	    {
	      
	      MYSQL_ROW row;
	      unsigned int i=0;
	      while ((row = mysql_fetch_row(result)))
		{
		  nodeid= atoi(row[0]);
		}		  
	      if(nodeid>0)
		{
  */
  
  snprintf(sql,sizeof(sql),"UPDATE mysql_server SET connected=1, role='%s' WHERE cid=%d and id=%d", role, cid,id);
		  //	  printf("SQL=%s\n", sql);
		  if(!executeQuery(sql,&mysql_conn[MYSQL_COLL_THREAD]))
		    {
		      log_message(LOG_WARNING, "Setting role on mysql_server table failed");		      
		    }	
		  /*
		}
	    }
	}
    }
  mysql_close(&mysql);
		  */
}



void insert_stats_tbl(unsigned int cid, unsigned int id, unsigned int nodeid)
{

  char sql[255];
  snprintf(sql,sizeof(sql),"INSERT IGNORE INTO  mysql_statistics(cid,id,nodeid) VALUES(%d ,%d, %d)", cid, id,nodeid);
  executeQuery(sql,&mysql_conn[MYSQL_COLL_THREAD]);
  
}

void insert_slave_stats_tbl(unsigned int cid, unsigned int id)
{

  char sql[255];
  snprintf(sql,sizeof(sql),"INSERT IGNORE INTO  mysql_slave_status(cid,id) VALUES(%d ,%d)", cid, id);
  executeQuery(sql,&mysql_conn[MYSQL_COLL_THREAD]);
  
}

void insert_master_stats_tbl(unsigned int cid, unsigned int id)
{

  char sql[255];
  snprintf(sql,sizeof(sql),"INSERT IGNORE INTO  mysql_master_status(cid,id) VALUES(%d ,%d)", cid, id);
  executeQuery(sql,&mysql_conn[MYSQL_COLL_THREAD]);
  
}

void collect_mysql_stats(
			 const int id,
		  	 const int nodeid,
			 const char * hostname,
			 const char * username, 
			 const char * password, 
			 const char * role, 
			 const int port)
{


  char msg[1024];
  MYSQL   mysql;
  bool ok=false;

  ok=connectMySQL3(&mysql, hostname, username, password,  port);

  if(ok)
    {
      
      char sqlbuf[2048];
      memset(sqlbuf,0,sizeof(sqlbuf));
      
      //   sprintf( sqlbuf,"SHOW GLOBAL STATUS");
      
      int n_status_vars=sizeof(status_vars)/sizeof(char*);
      char where[1536];
      memset(where,0, sizeof(where));
      char item[64];
      memset(item,0, sizeof(item));

      for (int i=0; i<n_status_vars;i++)
	{
	  if(i<(n_status_vars-1))
	    snprintf(item,sizeof(item), "'%s',", status_vars[i]);
	  else
	    snprintf(item,sizeof(item), "'%s'", status_vars[i]);
	  strcat(where,item);
	  if(strlen(where) > 1535)
	    abort();
	}
      snprintf( sqlbuf,
		sizeof(sqlbuf),
		"select VARIABLE_NAME, VARIABLE_VALUE from information_schema.global_status WHERE VARIABLE_NAME IN(%s)", where);
      
      
      if(mysql_real_query( &mysql, sqlbuf, strlen(sqlbuf) ))
	{
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg, sizeof(msg), "Query %s failed: Error: %s",
                  sqlbuf,
		  mysql_error(&mysql));
	  log_message(LOG_ERR, msg);
	  mysql_close(&mysql);
	  return;
	}
      
      else // query succeeded, process any data returned by it
	{
	  MYSQL_RES * result = mysql_store_result(&mysql);
	  if (result)  // there are rows
	    {
	      
	      MYSQL_ROW row;
	      unsigned int num_fields;
	      char  * key;
	      char * val;

	      num_fields = mysql_num_fields(result);
	      while ((row = mysql_fetch_row(result)))
		{
		  char msg[512];
		  memset(msg,0,sizeof(msg));
		  key=row[0];
		  val = row[1];
		  snprintf(msg,
			   sizeof(msg),
			  "REPLACE mysql_statistics(cid,id,nodeid,var,value,report_ts) VALUES(%d,%d, %d,'%s', %llu, now())",
			  g_clusterid, 			  
			  id,
			  nodeid,
			  key,
			  atoll(val));
		  
		  
		  if(!executeQuery(msg,&mysql_conn[MYSQL_COLL_THREAD]))
		   {
		     mysql_close(&mysql);
		     return;
		   }
		  memset(msg,0,sizeof(msg));
		  if(strcmp(key,"UPTIME")==0)
		    {
		      snprintf(msg,
			       sizeof(msg),
			      "INSERT INTO mysql_global_statistics(var,value,report_ts) VALUES('%s', %llu, now()) "
			      "ON DUPLICATE KEY UPDATE  value=%llu",
			      key,
			      atoll(val),
			      atoll(val));
		    }
		  else
		    {
		      snprintf(msg,
			      sizeof(msg),
			      "INSERT INTO mysql_global_statistics(var,value,report_ts) VALUES('%s', %llu, now()) "
			      "ON DUPLICATE KEY UPDATE  value=value+%llu",
			      key,
			      atoll(val),
			      atoll(val));

		    }
		  if(!executeQuery(msg,&mysql_conn[MYSQL_COLL_THREAD]))
		   {
		     mysql_close(&mysql);
		     return;
		   }
		  
		}
	      mysql_free_result(result);
	    }
	}
      update_status(g_clusterid,id, true, " ");
      mysql_close(&mysql);
    }
  else
    {
      char * msg = (char*)malloc(512 * sizeof(char));
      memset(msg,0,512*sizeof(char));
      snprintf(msg,sizeof(msg),"cmon could not connect to mysql server on %s@%s:%d (password=\'%s\').<br>Remove mysqld and check permissions and hostname.<br>Maybe you have forgot to GRANT?", username, hostname,port,password);
      update_status2(g_clusterid,hostname,port, false,msg);
      free(msg);
      mysql_close(&mysql);
    }
}
			 



void collect_mysql_slave_stats(const int id,
			       const char * hostname,
			       const char * username, 
			       const char * password, 
			       const int port,
			       const char * role)
{


  char msg[1024];
  MYSQL  mysql;
  bool ok=false;
  ok=connectMySQL3(&mysql, hostname, username, password,  port);
  if(ok)
    {
      
      char sqlbuf[255];
      memset(sqlbuf,0,sizeof(sqlbuf));

      snprintf( sqlbuf, sizeof(sqlbuf),"SHOW SLAVE STATUS");

      int a=strlen(sqlbuf);
      if(mysql_real_query( &mysql, sqlbuf, a ))
	{
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg,
		   sizeof(msg),
		   "Query %s failed: Error: %s",
		   sqlbuf,
		   mysql_error(&mysql));
	  log_message(LOG_ERR, msg);	 
	  mysql_close(&mysql);
	  char errmsg[1024];
	  memset(errmsg,0,sizeof(errmsg));
	  snprintf(errmsg, sizeof(errmsg),"UPDATE mysql_server SET msg='%s' WHERE cid=%d AND id=%d", msg, g_clusterid, id);
	  if(connectMySQL(&mysql_conn[MYSQL_COLL_THREAD_ERR]))
	    {
	      executeQuery(errmsg,&mysql_conn[MYSQL_COLL_THREAD_ERR]);  
	      mysql_close(&mysql_conn[MYSQL_COLL_THREAD_ERR].mysql);	      
	    }
	  return;
	}
      else // SHOW SLAVE STATUS succeeded, process any data returned by it
	{
	  MYSQL_RES * result = mysql_store_result(&mysql);
	  if (result)  // there are rows
	    {
	      
	      MYSQL_ROW row;
	      unsigned int num_fields;
	      char msg[64000];
	      char val[10240];	      
	      memset(val,0,sizeof(val));
	      memset(msg,0,sizeof(msg));
	      num_fields = mysql_num_fields(result);
	      unsigned long long num_rows = mysql_affected_rows(&mysql);
	      if((int)num_rows==0)
		{
		  /*
		    sprintf(msg,
			  "replace into mysql_slave_status (cid, id,report_ts) values (%d ,%d, NOW())", 
			  g_clusterid, id);
		  */
		}
	      else
		{
		  insert_slave_stats_tbl(g_clusterid,id);		      
		  //		  printf("was %s  - in slavee\n",role);
		  if(strcmp(role,"")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"slave");
		  if(strcmp(role,"mixed")==0)
		     update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"mixed");
		  if(strcmp(role,"master")==0)
		     update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"mixed");
		  if(strcmp(role,"slave")==0)
		     update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"slave");

		  unsigned int end=num_fields/2;
		  unsigned int j=0;

		  while ((row = mysql_fetch_row(result)))
		    {
		      while (1)
			{
			  snprintf(msg, 
				   sizeof(msg),
				  "UPDATE mysql_slave_status SET ");
			  
			  while(j<end)
			    {
			      memset(val,0,sizeof(val));
			      MYSQL_FIELD *field;
			      const char * filter1="Master_SSL_";
			      const char * filter2="Master_Bind";
			      field = mysql_fetch_field_direct(result, j);
			      if((strncmp(field->name,filter1,strlen(filter1))!=0) && 
				 (strncmp(field->name,filter2,strlen(filter2))!=0))
				{
				  
				  if((row[j] && strcmp(row[j],"") !=0)  || 
				     (strcmp(field->name,"Slave_IO_State") == 0) ||
				     (strcmp(field->name,"Last_IO_Error") == 0) ||
				     (strcmp(field->name,"Last_IO_Errno") == 0) ||
				     (strcmp(field->name,"Last_SQL_Error") == 0) ||
				     (strcmp(field->name,"Last_SQL_Errno") == 0))
				    snprintf(val,
					     sizeof(val),
					     "%s=\"%s\",",
					     field->name,
					     row[j]);
				}
			      strncat(msg,val, sizeof(val));
			      j++;
			    }			  
			  strcat(msg,"report_ts=now()");
			  char cond[64];
			  snprintf(cond,sizeof(cond)," WHERE cid=%d and id=%d", g_clusterid,id);
			  strcat(msg,cond);
			  //  printf("%s\n", msg);
			  executeQuery(msg,&mysql_conn[MYSQL_COLL_THREAD]);
			  end=num_fields;		      
			  if(j==end)
			    break;
			}
		    }
		}
	      mysql_free_result(result);
	      
	      /** GET NDB_APPLY_STATUS DATA **/

	      char sqlbuf[255];
	      memset(sqlbuf,0,sizeof(sqlbuf));
	      
	      snprintf( sqlbuf, sizeof(sqlbuf),"SELECT server_id, epoch FROM mysql.ndb_apply_status ORDER BY epoch desc LIMIT 1");
	      
	      int a=strlen(sqlbuf);
	      if(mysql_real_query( &mysql, sqlbuf, a ))
		{
		  
		  memset(msg,0,sizeof(msg));
		  snprintf(msg, 
			   sizeof(msg),
			   "Query %s failed: Error: %s",
			   sqlbuf,
			   mysql_error(&mysql));
		  log_message(LOG_ERR, msg);	 
		  mysql_close(&mysql);
		  char errmsg[1024];
		  memset(errmsg,0,sizeof(errmsg));
		  snprintf(errmsg,sizeof(errmsg),"UPDATE mysql_server SET msg='%s' WHERE cid=%d AND id=%d", msg, g_clusterid, id);
		  if(connectMySQL(&mysql_conn[MYSQL_COLL_THREAD_ERR]))
		    {
		      executeQuery(errmsg,&mysql_conn[MYSQL_COLL_THREAD_ERR]);  
		      mysql_close(&mysql_conn[MYSQL_COLL_THREAD_ERR].mysql);	      
		    }
		  return;
		}      
	      else // query succeeded, process any data returned by it
		{
		  MYSQL_RES * result = mysql_store_result(&mysql);
		  if (result)  // there are rows
		    {
		      
		      MYSQL_ROW row;
		      char msg[6400];
		      memset(msg,0,sizeof(msg));
		      unsigned long long num_rows = mysql_affected_rows(&mysql);
		      if((int)num_rows>0)
			{
			  unsigned long long epoch=0;
			  unsigned int serverid=0;
			  while ((row = mysql_fetch_row(result)))
			    {
			      serverid=atoi(row[0]);
			      epoch=atoll(row[1]);
			    }
			  snprintf(msg,
				   sizeof(msg),
				   "UPDATE mysql_slave_status SET epoch=%llu, serverid=%d WHERE cid=%d AND id=%d",  
				   epoch,
				   serverid,
				   g_clusterid,   
				   id);
			  executeQuery(msg,&mysql_conn[MYSQL_COLL_THREAD]); 		  
			}
		      mysql_free_result(result);
		    }
		}
	    }
	}
      update_status(g_clusterid,id, true, " ");   
      mysql_close(&mysql);
    }
  else
    {
      char * msg = (char*)malloc(512 * sizeof(char));
      memset(msg,0,512*sizeof(char));
      snprintf(msg, sizeof(msg),"cmon could not connect to mysql server on %s@%s:%d (password=\'%s\').<br>Remove mysqld and check permissions and hostname.<br>Maybe you have forgot to GRANT?", username, hostname,port,password);
      update_status(g_clusterid,id, false, msg);
      free (msg);
      mysql_close(&mysql);
    }
}
			 


void collect_mysql_master_stats(const int id,
				const char * hostname,
				const char * username, 
				const char * password, 
				const int port,
				const char * role)
{


  char msg[1024];
  MYSQL  mysql;
  bool ok=false;
  ok=connectMySQL3(&mysql, hostname, username, password,  port);
  if(ok)
    {
      
      char sqlbuf[255];
      memset(sqlbuf,0,sizeof(sqlbuf));

      snprintf( sqlbuf,sizeof(sqlbuf),"SHOW MASTER STATUS");

      int a=strlen(sqlbuf);
      if(mysql_real_query( &mysql, sqlbuf, a ))
	{
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s",
                  sqlbuf,
		  mysql_error(&mysql));
	  log_message(LOG_ERR, msg);	 
	  mysql_close(&mysql);
	  char errmsg[1024];
	  memset(errmsg,0,sizeof(errmsg));
	  snprintf(errmsg,sizeof(errmsg),"UPDATE mysql_server SET msg='%s' WHERE cid=%d AND id=%d", msg, g_clusterid, id);
	  if(connectMySQL(&mysql_conn[MYSQL_COLL_THREAD_ERR]))
	    {
	      executeQuery(errmsg,&mysql_conn[MYSQL_COLL_THREAD_ERR]);  
	      mysql_close(&mysql_conn[MYSQL_COLL_THREAD_ERR].mysql);	      
	    }
	  return;
	}
      else // SHOW MASTER STATUS succeeded, process any data returned by it
	{
	  MYSQL_RES * result = mysql_store_result(&mysql);
	  if (result)  // there are rows
	    {
	      
	      MYSQL_ROW row;
	      unsigned int num_fields;
	      char msg2[512];
	      num_fields = mysql_num_fields(result);
	      unsigned long long num_rows = mysql_affected_rows(&mysql);
	      if((int)num_rows==0)
		{
		  /*
		    sprintf(msg2,
			  "replace into mysql_slave_status (cid, id,report_ts) values (%d ,%d, NOW())", 
			  g_clusterid, id);
		  */
		  if(strcmp(role,"mixed")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"mixed");
		  if(strcmp(role,"none")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"none");

		}
	      else
		{
		  insert_master_stats_tbl(g_clusterid,id);		      
		  //		  printf("was %s  - in master\n",role);
		  if(strcmp(role,"")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"slave");
		  if(strcmp(role,"slave")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"mixed");
		  if(strcmp(role,"mixed")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"mixed");
		  if(strcmp(role,"master")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"master");
		  if(strcmp(role,"none")==0)
		    update_server_tbl_set_role(g_clusterid,id, hostname, username, password, port,"master");


		  while ((row = mysql_fetch_row(result)))
		    {
		      for(unsigned int j=0;j<num_fields;j++)
			{
			  memset(msg2,0,sizeof(msg2));
			  MYSQL_FIELD *field;
			  field = mysql_fetch_field_direct(result, j);
			  if(row[j] && strcmp(row[j],"") !=0)
			    {
			      snprintf(msg2, 
				       sizeof(msg2),
				       "UPDATE mysql_master_status SET %s='%s', report_ts=now() WHERE cid=%d AND id=%d",
				       field->name, row[j],g_clusterid,id);
			      if(!executeQuery(msg2,&mysql_conn[MYSQL_COLL_THREAD]))
				continue;
			    }
			}
		    }

		}	
	      mysql_free_result(result);
	    }
	}
      update_status(g_clusterid,id, true, " ");   
      mysql_close(&mysql);
    }
  else
    {
      char * msg = (char*)malloc(512 * sizeof(char));
      memset(msg,0,512*sizeof(char));
      snprintf(msg, sizeof(msg),"cmon could not connect to mysql server on %s@%s:%d (password=\'%s\').<br>Remove mysqld and check permissions and hostname.<br>Maybe you have forgot to GRANT?", username, hostname,port,password);
      update_status(g_clusterid,id, false, msg);
      free (msg);
      mysql_close(&mysql);
    }
}
			 



void collect_mysql_variables(const int mysqlid,
			     const char * hostname,
			     const char * username, 
			     const char * password, 
			     const char * role, 
			     const int port)
{


  char msg[1024];
  MYSQL  mysql;
  bool ok=false;
  ok=connectMySQL3(&mysql, hostname, username, password, port);

  if(ok)
    {      
      char sqlbuf[255];
      memset(sqlbuf,0,sizeof(sqlbuf));

      snprintf( sqlbuf, sizeof(sqlbuf),"SELECT VARIABLE_NAME, VARIABLE_VALUE FROM information_schema.global_variables");

      int a=strlen(sqlbuf);
      if(mysql_real_query( &mysql, sqlbuf, a ))
	{
	  
	  memset(msg,0,sizeof(msg));
	  snprintf(msg, 
		   sizeof(msg),
		   "Query %s failed: Error: %s",
		   sqlbuf,
		   mysql_error(&mysql));
	  log_message(LOG_ERR, msg);
	  mysql_close(&mysql);
	}
      
      else // query succeeded, process any data returned by it
	{
	  MYSQL_RES * result = mysql_store_result(&mysql);
	  if (result)  // there are rows
	    {
	      
	      MYSQL_ROW row;
	      unsigned int num_fields;
	      unsigned int i=0;
	      char msg[64000];
	      char val[512];	      
	      memset(msg,0,sizeof(msg));
	      memset(val,0,sizeof(val));
	      snprintf(msg,
		       sizeof(msg),
		      "replace into mysql_variables"
		      "(mysqlid, variable_name, variable_value) VALUES ");
	      
	      num_fields = mysql_num_fields(result);
	      unsigned long long num_rows = mysql_affected_rows(&mysql);
	      while ((row = mysql_fetch_row(result)))
		{
		  snprintf(val,
			   sizeof(val),
			   "(%d,'%s','%s')",
			   mysqlid,
			   row[0],
			   row[1]);
		  strncat(msg,val, sizeof(val));
		  i++;
		  if(i<num_rows)
		    strcat(msg,",");
		}
	      executeQuery(msg,&mysql_conn[MYSQL_COLL_THREAD]);
	      mysql_free_result(result);
	    }
	}    
      mysql_close(&mysql);
    }
}
		
	 
void update_status(unsigned int cid, unsigned int id, bool is_connected, const char * err)
{


  
  MYSQL * mysql;
  char msg[1024];
  
  mysql = &mysql_conn[MYSQL_COLL_THREAD].mysql;
  if(mysql_ping(mysql))
    {
      log_message(LOG_WARNING, "mysql_collector - lost connection to cmon mysql server");
      return;
    }
  
  char sqlbuf[768];
  memset(sqlbuf,0,sizeof(sqlbuf));
  snprintf( sqlbuf,sizeof(sqlbuf),"UPDATE mysql_server SET report_ts=now(), connected=%d, msg='%s' WHERE cid=%d and id=%d", 
	   is_connected, 
	   err,
	   cid,
	   id);

  int a=strlen(sqlbuf);
  //  printf("query=%s\n",sqlbuf);
  mysql=&mysql_conn[MYSQL_COLL_THREAD].mysql;
  if(mysql_real_query( mysql, sqlbuf, a ))
    {
            
      memset(msg,0,sizeof(msg));
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s",
	      sqlbuf,
	      mysql_error(mysql));
      log_message(LOG_ERR, msg);
      
    }       
}


void update_status2(unsigned int cid, const char * hostname, int port,  bool is_connected, const char * err)
{
  
  MYSQL * mysql;
  char msg[1024];
  
  mysql = &mysql_conn[MYSQL_COLL_THREAD].mysql;
  if(mysql_ping(mysql))
    {
      log_message(LOG_WARNING, "mysql_collector - lost connection to cmon mysql server");
      mysql_close(mysql);
      return;
    }
  
  char sqlbuf[768];
  memset(sqlbuf,0,sizeof(sqlbuf));
  snprintf( sqlbuf,sizeof(sqlbuf),"UPDATE mysql_server SET report_ts=now(),connected=%d, msg='%s' WHERE cid=%d AND port=%d AND hostname='%s'", 
	   is_connected, 
	   err,
	   cid,
	   port,
	   hostname);

  int a=strlen(sqlbuf);
  mysql=&mysql_conn[MYSQL_COLL_THREAD].mysql;
  if(mysql_real_query( mysql, sqlbuf, a ))
    {
      
      memset(msg,0,sizeof(msg));
      snprintf(msg,sizeof(msg), "Query %s failed: Error: %s",
	      sqlbuf,
	      mysql_error(mysql));
      log_message(LOG_ERR, msg);
      
    }       
}



