/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General  Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "mysql_coll.h"
#include "log.h"
#include "connect.h"
#include "statistics.h"


void checkRules(MYSQL * mysql, unsigned int mysql_id);
void checkThreadRule(MYSQL * mysql, unsigned int mysql_id);
void applyThreadRule(MYSQL * mysql, 
		     unsigned int mysql_id,
		     int threads_connected, 
		     int threads_cached);
int executeSelectQuery(MYSQL * mysql, const char * query);

void * mysql_monitor_thread(void * t)
{
  //configparam_t * params = (configparam_t*)t;
  log_message(LOG_INFO, "Starting MySQLMonitorThread");      
  MYSQL * mysql;
  
  int cycle=0;
  char msg[1024];
  while(1)
    {            
      sleep(30);
      if(!connectMySQL(&mysql_conn[MYSQL_MON_THREAD]))
	{
	  sleep(10);
	  continue;
	}  
      
      mysql=&mysql_conn[MYSQL_MON_THREAD].mysql;
      
      char sqlbuf[512];
      memset(sqlbuf,0,sizeof(sqlbuf));
      sprintf( sqlbuf,"SELECT id,hostname, username, password, port FROM mysql_server WHERE connected=1");
       int a=strlen(sqlbuf);
       if(mysql_real_query( mysql, sqlbuf, a ))
	 {
	   
	   memset(msg,0,sizeof(msg));
	   sprintf(msg, "Query %s failed: Error: %s",
                  sqlbuf,
		   mysql_error(mysql));
	   log_message(LOG_ERR, msg);
	   continue;
	 }       
       else // query succeeded, process any data returned by it
	 {
	   MYSQL_RES * result = mysql_store_result(mysql);
	   if (result)  // there are rows
	     {
	       MYSQL_ROW row;
	       unsigned int num_fields;
	       
	       num_fields = mysql_num_fields(result);
	       while ((row = mysql_fetch_row(result)))
		 {
		   checkRules(mysql, atoi(row[0]));
		 }	       
	       mysql_free_result(result);
	     }
	 }
       mysql_close(mysql);	       
       cycle++;
     }
}

void checkRules(MYSQL * mysql, unsigned int mysql_id)
{
  //  applyConnectionRule(mysql, mysql_id);
  checkThreadRule(mysql, mysql_id);
}

void checkThreadRule(MYSQL * mysql, unsigned int mysql_id)
{
       char sqlbuf[512];
       int THREADS_CACHED=0, THREADS_CONNECTED=0;
       
       memset(sqlbuf,0,sizeof(sqlbuf));
       sprintf( sqlbuf,"select THREADS_CACHED, THREADS_CONNECTED from mysql_statistics where mysqlid=%d order by report_ts desc limit 1", mysql_id);
              
       if(executeSelectQuery(mysql, sqlbuf))
	 {
	   return;
	 }
       else
	 {
	   MYSQL_RES * result = mysql_store_result(mysql);
	   if (result) 
	     {
	       MYSQL_ROW row;
	       while ((row = mysql_fetch_row(result)))
		 {
		   THREADS_CACHED=atoi(row[0]);
		   THREADS_CONNECTED=atoi(row[1]);		   
		 }	       	       
	       mysql_free_result(result);	       	       
	       applyThreadRule(mysql, 
			       mysql_id,
			       THREADS_CONNECTED,
			       THREADS_CACHED);
	     }
	 }       
}
 


int executeSelectQuery(MYSQL * mysql, const char * query)
{
  int ret=0;
  int len=strlen(query);
  if((ret=mysql_real_query( mysql, query, len )))
    {
      char msg[512];
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      query,
	      mysql_error(mysql));
      log_message(LOG_ERR, msg);
    }       
  return ret;
  
}

void applyThreadRule(MYSQL * mysql, 
		     unsigned int mysql_id,
		     int threads_connected, 
		     int threads_cached)
{

}
