/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "ndb_coll.h"
#include "log.h"
#include "connect.h"
#include "statistics.h"

extern Ndb_cluster_connection  * g_ndb_conn;

Ndb * ndb;
static const NdbDictionary::Dictionary * dic = 0;

static int _parsable = 0;
static int show_temp_status = 1;
extern int g_clusterid;

void * ndb_collector_thread(void * t)
{
  log_message(LOG_INFO, "Starting ndbd collector thread");    
  ndb=0;
 retry:
  if(ndb != 0)
    delete ndb;

  if(connect_ndb() <0)
    {
      sleep(10);
      goto retry;
    }

  ndb = new Ndb(g_ndb_conn, "cmon");
  
  if(ndb)
    {
      if(ndb->init())
	{
	  log_message(LOG_INFO,"Could not allocate ndb object..This is serious. Retrying..");
	  sleep(10);
	  goto retry;
	}
      
      log_message(LOG_INFO,"Connected to Cluster.");  
    }
  else
    {
      log_message(LOG_INFO,"Could not allocate ndb object..This is serious. Retrying..");
      sleep(10);
      goto retry;
    }

  
  while(1)
    {      
      sleep(60);      
      if(!connectMySQL(&mysql_conn[NDB_COLL_THREAD]))
	{
	  sleep(10);
	  continue;
	}
      if( get_diskdata_stats(ndb) < 0)
	{
	  disconnectMySQL(&mysql_conn[NDB_COLL_THREAD]);
	  goto retry;
	}
      disconnectMySQL(&mysql_conn[NDB_COLL_THREAD]);
      sleep(10);      
      if(!connectMySQL(&mysql_conn[NDB_COLL_THREAD]))
	{
	  sleep(10);
	  continue;
	}

      if( list_tables(0, 
		      (NdbDictionary::Object::Type)0) <0)
	{
	  disconnectMySQL(&mysql_conn[NDB_COLL_THREAD]);
	  goto retry;
	}
      disconnectMySQL(&mysql_conn[NDB_COLL_THREAD]);
    }
  return 0;
}


int  get_diskdata_stats(Ndb  * ndb)
{


  
  if(ndb==0)
    {
      log_message(LOG_WARNING,"ndb object is null...");    
      return -1;
    }

  Ndb_cluster_connection *conn= g_ndb_conn;
  
  NdbDictionary::Dictionary* dict= ndb->getDictionary();
  if(dict==0)
    {
       log_message(LOG_WARNING,"erroor...");    
       return -1;
     }


  NdbDictionary::Dictionary::List dflist;
  NdbError ndberr;
  uint i;

  dict->listObjects(dflist, NdbDictionary::Object::Datafile);
  ndberr= dict->getNdbError();
  if (ndberr.classification != NdbError::NoError)
    {
      log_message(LOG_WARNING,"error...");
      return -1;
    }
  char str[1024];
  memset(str,0,sizeof(str));
  sprintf(str,"DELETE FROM diskdata");
  executeQuery(str, &mysql_conn[NDB_COLL_THREAD]);

  for (i= 0; i < dflist.count; i++)
  {
    NdbDictionary::Dictionary::List::Element& elt = dflist.elements[i];
    Ndb_cluster_connection_node_iter iter;
    uint id;    
    conn->init_get_next_node(iter);

    while ((id= conn->get_next_node(iter)))
    {
      NdbDictionary::Datafile df= dict->getDatafile(id, elt.name);
      ndberr= dict->getNdbError();
      if(ndberr.classification != NdbError::NoError)
      {
        if (ndberr.classification == NdbError::SchemaError)
          continue;

        if (ndberr.classification == NdbError::UnknownResultError)
          continue;
	log_message(LOG_WARNING,"erroor..111.");
	return -1;   
      }
      NdbDictionary::Tablespace ts= dict->getTablespace(df.getTablespace());
      ndberr= dict->getNdbError();
      if (ndberr.classification != NdbError::NoError)
      {
        if (ndberr.classification == NdbError::SchemaError)
          continue;
	log_message(LOG_WARNING,"erroor..333.");
	return -1;
      }
      int size=	      ts.getExtentSize();
      
      memset(str,0,sizeof(str));
      sprintf(str,
	      "REPLACE INTO cmon.diskdata(cid,nodeid, file_name,"
	      "type, ts_name, logfile_group_name, free_extents, total_extents,"
	      "extent_size, initial_size, maximum_size) VALUES"
	      "(%d,%d, '%s', '%s','%s', '%s', %llu, %llu, %d, %llu, %llu )", 
	      g_clusterid,
	      id,
	      elt.name,
	      "DATAFILE",
	      df.getTablespace(),
	      ts.getDefaultLogfileGroup(),
	      (df.getFree()/ size),
	      (df.getSize()/ size),
	      ts.getExtentSize(),
	      df.getSize(),
	      df.getSize());
      executeQuery(str, &mysql_conn[NDB_COLL_THREAD]);
    }
  }
  
  NdbDictionary::Dictionary::List uflist;
  dict->listObjects(uflist, NdbDictionary::Object::Undofile);
  ndberr= dict->getNdbError();
  if (ndberr.classification != NdbError::NoError)
    {
      log_message(LOG_WARNING,"erroor..111.");
      return -1;
    }      
  
  for (i= 0; i < uflist.count; i++)
      {
	NdbDictionary::Dictionary::List::Element& elt= uflist.elements[i];
	Ndb_cluster_connection_node_iter iter;
	unsigned id;
	
	g_ndb_conn->init_get_next_node(iter);
	
	while ((id= g_ndb_conn->get_next_node(iter)))
	  {
	    NdbDictionary::Undofile uf= dict->getUndofile(id, elt.name);
	    ndberr= dict->getNdbError();
	    if (ndberr.classification != NdbError::NoError)
	      {
		if (ndberr.classification == NdbError::SchemaError)
		  continue;
		if (ndberr.classification == NdbError::UnknownResultError)
          continue;
		
		log_message(LOG_WARNING,"erroor..111.");
		return -1;	      
		
	      }
	    NdbDictionary::LogfileGroup lfg=
	      dict->getLogfileGroup(uf.getLogfileGroup());
	    ndberr= dict->getNdbError();
	    if (ndberr.classification != NdbError::NoError)
	      {
		if (ndberr.classification == NdbError::SchemaError)
		  continue;
		log_message(LOG_WARNING,"erroor..111.");
		return -1;	      
	      }
	    
	    NdbDictionary::ObjectId objid;
	    uf.getLogfileGroupId(&objid);
	    int x=  uf.getSize();
	    sprintf(str,
		    "REPLACE INTO cmon.diskdata(cid, nodeid, file_name,"
		    "type, logfile_group_name,  total_extents,"
		    "extent_size, initial_size, maximum_size, "
		    "undo_buffer_size) VALUES"
		    "(%d, %d, '%s','%s', '%s', %d, 4 ,%d ,%d,%lu)", 
		    g_clusterid,
		    id,
		    elt.name,
		    "UNDO_LOG",
		    uf.getLogfileGroup(),
		    x/4,
		    x,
		    x,
		    (unsigned long) lfg.getUndoBufferSize());
	    executeQuery(str, &mysql_conn[NDB_COLL_THREAD]);
	  }
	
      }
  NdbDictionary::Dictionary::List lfglist;
  dict->listObjects(lfglist, NdbDictionary::Object::LogfileGroup);
  ndberr= dict->getNdbError();
  if (ndberr.classification != NdbError::NoError)
    {
     	log_message(LOG_WARNING,"erroor..111.");
	return -1;
    }

  for (i= 0; i < lfglist.count; i++)
    {
      NdbDictionary::Dictionary::List::Element& elt= lfglist.elements[i];
      
      NdbDictionary::LogfileGroup lfg= dict->getLogfileGroup(elt.name);
      ndberr= dict->getNdbError();
      if (ndberr.classification != NdbError::NoError)
	{
	  if (ndberr.classification == NdbError::SchemaError)
	    continue;
	  {
	    log_message(LOG_WARNING,"erroor..111.");
	    return -1;
	  }
	}
      
      sprintf(str,
	      "REPLACE INTO cmon.diskdata(cid, nodeid, file_name,"
	      "type, logfile_group_name, free_extents, "
	      "undo_buffer_size) VALUES"
	      "(%d,%d, '%s','%s','%s',  %llu,%lu)", 
	      g_clusterid,
	      99,
	      elt.name,
	      "LOGFILE_GROUP",
	      lfg.getName(),
	      lfg.getUndoFreeWords(),
	      (unsigned long) lfg.getUndoBufferSize());
      
      executeQuery(str, &mysql_conn[NDB_COLL_THREAD]);
	    
    }
  return 1;
}





int list_tables(const char * tabname, 
	  NdbDictionary::Object::Type type)
{
  char str[1024];
  memset(str,0,1024);
  dic= ndb->getDictionary();
  NdbDictionary::Dictionary::List list;
  if (tabname == 0) {
    if (dic->listObjects(list,(NdbDictionary::Object::Type)0 ) == -1)
      {
       printf("blabla\n");
       return -1;
      }
  } else {
    if (dic->listIndexes(list, tabname) == -1)
      {
       printf("blabla\n");
       return -1;
      }
  }

  sprintf(str, "DELETE FROM schema_object WHERE cid=%d",g_clusterid);
  executeQuery(str, &mysql_conn[NDB_COLL_THREAD]);
  memset(str,0,1024);
  for (unsigned i = 0; i < list.count; i++) {
    NdbDictionary::Dictionary::List::Element& elt = list.elements[i];
    char type[100];
	bool isTable = false;
        switch (elt.type) {
        case NdbDictionary::Object::SystemTable:
            strcpy(type, "SystemTable");
	    isTable = true;
            break;
        case NdbDictionary::Object::UserTable:
            strcpy(type, "UserTable");
	    isTable = true;
            break;
        case NdbDictionary::Object::UniqueHashIndex:
            strcpy(type, "UniqueHashIndex");
	    isTable = true;
            break;
        case NdbDictionary::Object::OrderedIndex:
            strcpy(type, "OrderedIndex");
	    isTable = true;
            break;
        case NdbDictionary::Object::HashIndexTrigger:
            strcpy(type, "HashIndexTrigger");
            break;
        case NdbDictionary::Object::IndexTrigger:
            strcpy(type, "IndexTrigger");
            break;
        case NdbDictionary::Object::SubscriptionTrigger:
            strcpy(type, "SubscriptionTrigger");
            break;
        case NdbDictionary::Object::ReadOnlyConstraint:
            strcpy(type, "ReadOnlyConstraint");
            break;
	case NdbDictionary::Object::Tablespace:
	  strcpy(type, "Tablespace");
	  break;
	case NdbDictionary::Object::LogfileGroup:
	  strcpy(type, "LogfileGroup");
	  break;
	case NdbDictionary::Object::Datafile:
	  strcpy(type, "Datafile");
	  break;
	case NdbDictionary::Object::Undofile:
	  strcpy(type, "Undofile");
	  break;
        case NdbDictionary::Object::TableEvent:
            strcpy(type, "TableEvent");
            break;
        default:
	  sprintf(type, "%d", (int)elt.type);
            break;
        }
        char state[100];
        switch (elt.state) {
        case NdbDictionary::Object::StateOffline:
            strcpy(state, "Offline");
            break;
        case NdbDictionary::Object::StateBuilding:
            strcpy(state, "Building");
            break;
        case NdbDictionary::Object::StateDropping:
            strcpy(state, "Dropping");
            break;
        case NdbDictionary::Object::StateOnline:
            strcpy(state, "Online");
            break;
        case NdbDictionary::Object::StateBackup:
	    strcpy(state, "Backup");
            break;
        case NdbDictionary::Object::StateBroken:
            strcpy(state, "Broken");
            break;
        default:
            sprintf(state, "%d", (int)elt.state);
            break;
        }
        char store[100];
	if (! isTable)
	    strcpy(store, "-");
	else {
	    switch (elt.store) {
	    case NdbDictionary::Object::StoreNotLogged:
		strcpy(store, "No");
		break;
	    case NdbDictionary::Object::StorePermanent:
		strcpy(store, "Yes");
		break;
	    default:
		sprintf(store, "%d", (int)elt.store);
		break;
	    }
	}
        char temp[100];
        if (show_temp_status)
        {
          if (! isTable)
              strcpy(temp, "-");
          else {
              switch (elt.temp) {
              case NDB_TEMP_TAB_PERMANENT:
                  strcpy(temp, "No");
                  break;
              case NDB_TEMP_TAB_TEMPORARY:
                  strcpy(temp, "Yes");
                  break;
              default:
                  sprintf(temp, "%d", (int)elt.temp);
                  break;
              }
          }
        }
	bool ok=true;
	if((strncmp(elt.name,"REPL",4) == 0) || strncmp(elt.name,"NDB",3) ==  0)
	  ok =false;
	  
	if(ok)
	  {
	    if (ndb->usingFullyQualifiedNames())
	      {
		
		if (_parsable)
		  {	
		    sprintf(str,
			    "INSERT INTO schema_object("
			    "cid, tableid,type,state,logging,temp,db,schema_name,name) VALUES"
			    "(%d, %d,'%s','%s','%s','%s','%s','%s','%s')",g_clusterid, elt.id, type, state, store, temp, 
			    (elt.database)?elt.database:"", (elt.schema)?elt.schema:"", elt.name);
		  }
		
		
		else
		  {
		    sprintf(str,
			    "INSERT INTO schema_object("
			    "cid,tableid,type,state,logging, temp, db,schema_name, name) VALUES"
			    "(%d,%d,'%s','%s','%s','%s','%s','%s','%s')", g_clusterid, elt.id, type, state, store, temp, 
			    (elt.database)?elt.database:"", (elt.schema)?elt.schema:"", elt.name);
		  }
	      }
	    else
	      {
		if (_parsable)
		  {
		    
		    sprintf(str,
			    "INSERT INTO schema_object("
			    "cid,tableid,type,state,logging,temp, name) VALUES"
			    "(%d,%d,'%s','%s','%s','%s','%s')", g_clusterid, elt.id, type, state, store, temp, elt.name);
		  }
		else
		  {
		    sprintf(str,
			    "INSERT INTO schema_object("
			    "cid,tableid,type,state,logging,temp, name) VALUES"
			    "(%d,%d,'%s','%s','%s','%s','%s')", g_clusterid, elt.id, type, state, store, temp, elt.name);
		  }
	      }	  
	    executeQuery(str, &mysql_conn[NDB_COLL_THREAD]);
	  }
    }
  return 1;
}

