/* Copyright (C) 2008 severalnines.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#include <mysql.h>
#include <stdio.h>
#include "connect.h"
#include "schema.h"
#include "log.h"
#include "mysql_coll.h"

bool checkSchema()
{
  
  char sqlbuf[128];

  char msg[16384];
      
  MYSQL  mysql;

  if(!connectMySQL2(&mysql, ""))
    {
      //      log_message(LOG_ERR, 
      fprintf(stderr,"Check if mysql server is started or that cmon user has correct privileges!\nTerminating..\n");
      cmon_exit(-1);
    }
  
  memset(sqlbuf,0,sizeof(sqlbuf));

  sprintf( sqlbuf, "CREATE DATABASE IF NOT EXISTS %s", params.db );
  if(mysql_query( &mysql, sqlbuf ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
              sqlbuf,
              mysql_error(&mysql));
      fprintf(stderr,"%s\n",msg);
      //      log_message(LOG_ERR, msg);
      cmon_exit(mysql_errno(&mysql));
    }

  memset(sqlbuf,0,sizeof(sqlbuf));
  sprintf( sqlbuf, "USE %s", params.db );

      if(mysql_query(&mysql, sqlbuf ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
              sqlbuf,
              mysql_error(&mysql));
      log_message(LOG_ERR, msg);
      cmon_exit(mysql_errno(&mysql));
    }

  memset(sqlbuf,0,sizeof(sqlbuf));

  MYSQL_RES *result;

  int i=0;
  while(i<NO_TABLES )
    {
      memset(sqlbuf,0,sizeof(sqlbuf));
      sprintf( sqlbuf,
	       "select TABLE_NAME from information_schema.tables where TABLE_SCHEMA='cmon' and TABLE_NAME='%s'",TABLES[i] );
      int a=strlen(sqlbuf);
      if(mysql_real_query( &mysql, sqlbuf, a ))
        {
	  memset(msg,0,sizeof(msg));
          sprintf(msg, "Query %s failed: Error: %s",
                  sqlbuf,
                  mysql_error(&mysql));
          log_message(LOG_ERR, msg);
	  cmon_exit(mysql_errno(&mysql));
        }

      else // query succeeded, process any data returned by it
        {
          result = mysql_store_result(&mysql);
          if (result)  // there are rows
            {
              unsigned long long num_rows = mysql_affected_rows(&mysql);

	      mysql_free_result(result);

              if(num_rows==0)
                {
		  memset(msg,0,sizeof(msg));
                  sprintf(msg,
			  "Table '%s' did not exist. Schema incomplete. Going to recreate missing tables.",
                          TABLES[i]);
                  log_message(LOG_INFO,msg);
		  mysql_close(&mysql);
                  return false;
               }
              else
                {

		  memset(msg,0,sizeof(msg));
                  sprintf(msg,"Table '%s' exists", TABLES[i]);
                  log_message(LOG_INFO,msg);
                }
            }
          else
            {
	      memset(msg,0,sizeof(msg));
              sprintf(msg,
		      "Table '%s' did not exist. Schema incomplete. Going to recreate missing tables.",
                      TABLES[i]);
              log_message(LOG_INFO,msg);
              mysql_close(&mysql);
              return false;
            }
        } 
      i++;
    }
  log_message(LOG_INFO,"Checked all tables");
  mysql_close(&mysql);

  return true;
}

void createSchema(bool drop)
{

  char sqlbuf[65536];
  memset(sqlbuf,0,sizeof(sqlbuf));
  char msg[65536];
  memset(msg,0,sizeof(msg));
  
  MYSQL mysql;
  if(mysql_init(&mysql)<0)
    return;

  
  if(!mysql_real_connect(&mysql, 
			 params.host, 
			 params.user, 
			 params.passwd, 
			 "", 
			 params.port, 
			 params.socket,
			 0))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      sqlbuf,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      cmon_exit(mysql_errno(&mysql));
      return;
    }
  

  sprintf( sqlbuf, "CREATE DATABASE IF NOT EXISTS %s", params.db );
  if(mysql_query( &mysql, sqlbuf ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      sqlbuf,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      cmon_exit(mysql_errno(&mysql));
      return ;
    }
  
  memset(sqlbuf,0,sizeof(sqlbuf));
  sprintf( sqlbuf, "USE %s", params.db );
  if(mysql_query( &mysql, sqlbuf ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      sqlbuf,
	      mysql_error(&mysql));
      
      log_message(LOG_ERR,msg);     
      cmon_exit(mysql_errno(&mysql));
    }
  

  if(drop)
    {
      if(mysql_query( &mysql, DROP_CLUSTER_LOG ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_CLUSTER_LOG,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);
	  
	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'cluster_log'");

      if(mysql_query( &mysql, DROP_MAILSERVER ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MAILSERVER,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);
	  
	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mailserver'");

      if(mysql_query( &mysql, DROP_EMAIL_NOTIFICATION))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_EMAIL_NOTIFICATION,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);
	  
	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'email_notification'");
      
      
      if(mysql_query( &mysql, DROP_CLUSTER_STATE ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_CLUSTER_STATE,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      
      log_message(LOG_INFO,"Dropped table 'cluster_state'");
  
      if(mysql_query( &mysql, DROP_MEMORY_USAGE ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MEMORY_USAGE,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'memory_usage'");


      if(mysql_query( &mysql, DROP_ALARM ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_ALARM,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'alarm'");


      if(mysql_query( &mysql, DROP_ALARM_LOG ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_ALARM_LOG,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'alarm_log'");
      
      
      if(mysql_query( &mysql, DROP_BACKUP ))
	{

	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_BACKUP,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}
      log_message(LOG_INFO,"Dropped table 'backup'");


      if(mysql_query( &mysql, DROP_SCHEMA_OBJECT ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_SCHEMA_OBJECT,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}
      log_message(LOG_INFO,"Dropped table 'schema_object'");


      if(mysql_query( &mysql, DROP_BACKUP_LOG ))
	{

	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_BACKUP_LOG,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}
      log_message(LOG_INFO,"Dropped table 'backup_log'");
      

      if(mysql_query( &mysql, DROP_RESTORE ))
	{

	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_RESTORE,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}
      log_message(LOG_INFO,"Dropped table 'restore'");


      if(mysql_query( &mysql, DROP_RESTORE_LOG ))
	{

	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_RESTORE_LOG,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}
      log_message(LOG_INFO,"Dropped table 'restore_log'");


      if(mysql_query( &mysql, DROP_NODE_STATE ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_NODE_STATE ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);
	  

	  cmon_exit(mysql_errno(&mysql));
	}  

      log_message(LOG_INFO,"Dropped table 'node_state'");
      
      if(mysql_query( &mysql, DROP_NODE_STATISTICS ))
	{
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_NODE_STATISTICS ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'node_statistics'");

      
      if(mysql_query( &mysql, DROP_CLUSTER_STATISTICS ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_CLUSTER_STATISTICS ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'cluster_statistics'");

      
      if(mysql_query( &mysql, DROP_CLUSTER ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_CLUSTER ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'cluster'");

      if(mysql_query( &mysql, DROP_CONFIGURATOR_NODEMAP ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_CONFIGURATOR_NODEMAP ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'configurator_nodemap'");


      if(mysql_query( &mysql, DROP_MYSQL_STATISTICS ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MYSQL_STATISTICS ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mysql_statistics'");
      


      if(mysql_query( &mysql, DROP_MYSQL_GLOBAL_STATISTICS ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MYSQL_GLOBAL_STATISTICS ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mysql_global_statistics'");
      


      
      if(mysql_query( &mysql, DROP_MYSQL_VARIABLE ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MYSQL_VARIABLE ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mysql_variable'");



      
      if(mysql_query( &mysql, DROP_MYSQL_SERVER ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MYSQL_SERVER ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mysql_server'");


      if(mysql_query( &mysql, DROP_MYSQL_SLAVE_STATUS ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MYSQL_SLAVE_STATUS ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mysql_slave_statistics'");



      if(mysql_query( &mysql, DROP_MYSQL_MASTER_STATUS ))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_MYSQL_MASTER_STATUS ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mysql_master_statistics'");


      if(mysql_query( &mysql, DROP_DISKDATA))
      {
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  DROP_DISKDATA ,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);

	  cmon_exit(mysql_errno(&mysql));
	}  
      log_message(LOG_INFO,"Dropped table 'mysql_server'");



    }
          


  if(mysql_query( &mysql, CREATE_BACKUP ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_BACKUP ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'backup'");


  if(mysql_query( &mysql, CREATE_MAILSERVER ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_MAILSERVER ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'mailserver'");


  if(mysql_query( &mysql, CREATE_EMAIL_NOTIFICATION ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_EMAIL_NOTIFICATION ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'email_notification'");


  if(mysql_query( &mysql, CREATE_ALARM ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_ALARM ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'alarm'");


  if(mysql_query( &mysql, CREATE_ALARM_LOG ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_ALARM_LOG ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'alarm_log'");

  if(mysql_query( &mysql, CREATE_CLUSTER ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_CLUSTER ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'cluster'");


  if(mysql_query( &mysql, CREATE_CONFIGURATOR_NODEMAP ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_CONFIGURATOR_NODEMAP ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'configurator_nodemap'");
  
  if(mysql_query( &mysql, CREATE_BACKUP_LOG ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_BACKUP_LOG ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,"Created table 'backup_log'");
  

  if(mysql_query( &mysql, CREATE_RESTORE ))
    {
      
	  memset(msg,0,sizeof(msg));
	  sprintf(msg, "Query %s failed: Error: %s",
		  CREATE_RESTORE,
		  mysql_error(&mysql));
	  log_message(LOG_ERR,msg);
	  
	  cmon_exit(mysql_errno(&mysql));
    }
  else
    log_message(LOG_INFO,"Created table 'restore'");
  
  
  if(mysql_query( &mysql, CREATE_RESTORE_LOG ))
    {
      
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_RESTORE_LOG,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }
  else
    log_message(LOG_INFO,"Created table 'restore_log'");
  


  if(mysql_query( &mysql, CREATE_CLUSTER_LOG ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_CLUSTER_LOG,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  else
    log_message(LOG_INFO,"Created table 'cluster_log'");



  if(mysql_query( &mysql, CREATE_CLUSTER_STATE ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_CLUSTER_STATE,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  else
    log_message(LOG_INFO,"Created table 'cluster_state'");

  if(mysql_query( &mysql, CREATE_MEMORY_USAGE ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_MEMORY_USAGE,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  else
    log_message(LOG_INFO,"Created table 'memory_usage'");


  if(mysql_query( &mysql, CREATE_NODE_STATE ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_NODE_STATE,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  else
    log_message(LOG_INFO,"Created table 'node_state'");



  if(mysql_query( &mysql, CREATE_CLUSTER_STATISTICS ))
    {      
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_CLUSTER_STATISTICS ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);

      cmon_exit(mysql_errno(&mysql));
    }
  else
    log_message(LOG_INFO,"Created table 'cluster_statistics'");



  if(mysql_query( &mysql, CREATE_NODE_STATISTICS ))
    {      
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_NODE_STATISTICS ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }
  log_message(LOG_INFO,(char*)"Created table 'node_statistics'");

  
  if(mysql_query( &mysql, CREATE_MYSQL_SLAVE_STATUS ))
    {
      memset(msg,0,sizeof(msg));
      snprintf(msg,512, 	  
	      "Query %s failed: Error: %s",
	       CREATE_MYSQL_SLAVE_STATUS ,
	       mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  log_message(LOG_INFO,"Created table 'mysql_slave_status'");
  
  if(mysql_query( &mysql, CREATE_MYSQL_MASTER_STATUS ))
    {
      memset(msg,0,sizeof(msg));
      snprintf(msg,512, 	  
	      "Query %s failed: Error: %s",
	       CREATE_MYSQL_MASTER_STATUS ,
	       mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  log_message(LOG_INFO,"Created table 'mysql_master_status'");
  

  if(mysql_query( &mysql, CREATE_MYSQL_VARIABLE ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_MYSQL_VARIABLE ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  log_message(LOG_INFO,"Created table 'mysql_variable'"); 
  
  if(mysql_query( &mysql, CREATE_MYSQL_SERVER ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_MYSQL_SERVER ,
		  mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  log_message(LOG_INFO,"Created table 'mysql_server'");

  
  char * stats_tbl=(char*)malloc(65536*sizeof(char));
  memset(stats_tbl,0,sizeof(stats_tbl));
  sprintf(stats_tbl, "CREATE TABLE IF NOT EXISTS `mysql_statistics` (`cid`INTEGER NOT NULL,`id`INTEGER NOT NULL,`nodeid`INTEGER NOT NULL,");
  int n_status_vars=sizeof(status_vars)/sizeof(char*);
  char item[512];

  for (int i=0; i<n_status_vars;i++)
    {
      memset(item,0,sizeof(item));
      if(i<(n_status_vars-1))
	snprintf(item,sizeof(item), "%s BIGINT DEFAULT '0',", status_vars[i]);
      else
	snprintf(item,sizeof(item), "%s BIGINT DEFAULT '0', "
		 "`report_ts` TIMESTAMP DEFAULT CURRENT_TIMESTAMP ," 
		 "PRIMARY KEY (`cid`,`id`)) ENGINE=MyISAM DEFAULT CHARSET=latin1", status_vars[i]);      
      strcat(stats_tbl,item);
    }



  if(mysql_query( &mysql, CREATE_MYSQL_STATISTICS ))
  //  if(mysql_query( &mysql, stats_tbl ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_MYSQL_STATISTICS ,
		  mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  free(stats_tbl);
  log_message(LOG_INFO,"Created table 'mysql_statistics'");


  if(mysql_query( &mysql, CREATE_MYSQL_GLOBAL_STATISTICS ))
  //  if(mysql_query( &mysql, stats_tbl ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_MYSQL_GLOBAL_STATISTICS ,
		  mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  log_message(LOG_INFO,"Created table 'mysql_global_statistics'");

  if(mysql_query( &mysql, CREATE_DISKDATA ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_DISKDATA ,
		  mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  log_message(LOG_INFO,"Created table 'diskdata'");


  if(mysql_query( &mysql, CREATE_SCHEMA_OBJECT ))
    {
      memset(msg,0,sizeof(msg));
      sprintf(msg, "Query %s failed: Error: %s",
	      CREATE_SCHEMA_OBJECT ,
	      mysql_error(&mysql));
      log_message(LOG_ERR,msg);
      
      cmon_exit(mysql_errno(&mysql));
    }  
  log_message(LOG_INFO,"Created table 'schema_object'");
   
  mysql_close(&mysql);
}

